function regder2_0v5 %CAU Kiel & UK Bratislava, January 2010, Auhtors: R.Pasteka & D.Kusnirak
%MAIN FUNCTION
clear all;close all;clc

%main window inicialization
mainwindow=figure('Name','REGDER2_0v4','NumberTitle','off');
bg=get(mainwindow,'Color');
[pic,map]=imread('main_logo.rlg','BackgroundColor',bg);
image(pic);colormap(map);axis image;axis off

set((gca(mainwindow)),'position',[0.001 0.001 1 1])

%main window parameters
set(mainwindow,'Menubar','none','Resize','off','Position',[410,450,560,200])

%navigation bar set up
menu1 = uimenu('Label','Profile data');
        uimenu(menu1,'Label','Open profile data file','Callback',@Open_profile_data_Callback);
        uimenu(menu1,'Label','Display profile data','Callback',@Display_profile_data_Callback);
        uimenu(menu1,'Label','Norm for x-derivative','Callback',@x_derivat_norm_2D_Callback,'Separator','on');
        uimenu(menu1,'Label','Norm for z-derivative','Callback',@z_derivat_norm_2D_Callback);
        uimenu(menu1,'Label','x-derivative calculation','Callback',@x_derivat_calc_2D_Callback,'Separator','on');
        uimenu(menu1,'Label','z-derivative calculation','Callback',@z_derivat_calc_2D_Callback);                        
                 
menu2 = uimenu('Label','Grid data');
        uimenu(menu2,'Label','Open grid data file','Callback',@Open_grid_data_Callback);
        uimenu(menu2,'Label','Display grid data','Callback',@Display_grid_data_Callback);
        uimenu(menu2,'Label','Norm for x-derivative','Callback',@x_derivat_norm_3D_Callback,'Separator','on');
        uimenu(menu2,'Label','Norm for y-derivative','Callback',@y_derivat_norm_3D_Callback);
        uimenu(menu2,'Label','Norm for z-derivative','Callback',@z_derivat_norm_3D_Callback);
        uimenu(menu2,'Label','x-derivative calculation','Callback',@x_derivat_calc_3D_Callback,'Separator','on');
        uimenu(menu2,'Label','y-derivative calculation','Callback',@y_derivat_calc_3D_Callback);
        uimenu(menu2,'Label','z-derivative calculation','Callback',@z_derivat_calc_3D_Callback);
                 
menu3 = uimenu('Label','Profile Euler deconvolution');
        uimenu(menu3,'Label','Set calcul. param. and START','Callback',@Euler_deconvolution_2D_Callback);
        uimenu(menu3,'Label','Select 2D solutions','Callback',@select_solutions_2D_Callback);
        
menu4 = uimenu('Label','Grid Euler deconvolution');
        uimenu(menu4,'Label','Set calcul. param. and START','Callback',@Euler_deconvolution_3D_Callback);
        uimenu(menu4,'Label','Select 3D solutions','Callback',@select_solutions_3D_Callback);

menu5 = uimenu('Label','End');
        uimenu(menu5,'Label','About','Callback',@about_Callback);   
        uimenu(menu5,'Label','Quit inside MATLAB','Callback',@matlab_exit_Callback');
        uimenu(menu5,'Label','Quit','Callback','exit','Accelerator','Q');  

        
%====================================CALLBACKS=============================%
%1. PROFILE DATA~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

function Open_profile_data_Callback(src,eventdata)
global X F extrapolF m numextrPoints pointsnextr fnameprofile pname_prof c_dir_prof

% INPUT OF PROFILE DATA - from 2 columns file (into matrix inpXdT)
c_dir_prof=pwd;
[fnameprofile,pname_prof]=uigetfile('*.dat','input of profile data (*.DAT file with 2 columns)');
%change the directory if needed and open the file
cd(pname_prof);  inpXF = dlmread(fnameprofile);
cd(c_dir_prof);
% columns of the readed matrix inpXF are separated into vectors X and dT
X=inpXF(:,1);  F=inpXF(:,2);
% number of readed data-rows is determined - variable m
m = numel(X); s = ['  number of readed profile data: ' int2str(m) '                                     (select next items from the "Profile data regul.deriv." menu)'];
%profile extrapolation - 15% each edge (to exclude the edge effects during FFT)
[extrapolF,numextrPoints] = cosxp2prof(F,0.15);
pointsnextr = numel(extrapolF);
%Display data and show info
Display_profile_data_Callback
msgbox(s,'message');

function Display_profile_data_Callback(src,eventdata)
global X F extrapolF m numextrPoints pointsnextr orig_fig

    % plotting the graph of the readed input data
    orig_fig=figure('Name','original data (2D)','NumberTitle','off');
    plot(X,F);
    % axes properties
    xlabel('x [dist. units]','FontSize',10); ylabel('input function [nT (or) mGal]','FontSize',10);
    legend('original data');
    title('input - original imported profile data   (select next items from "profile" menu)');
    set(orig_fig,'WindowButtonMotionFcn',@cursorcoordinate)

% --------------------------------------------------------------------  
function x_derivat_norm_2D_Callback(src,eventdata)
global X F m optalphax extrapolF numextrPoints pointsnextr fnameprofile pname_prof c_dir_prof

%initial check
if isempty(extrapolF)
    helpdlg('Load profile data first','Wrong selection');
    return
else
end
%%%%%%%% EVALUATION OF THE NORM FOR THE REGULARIZED X-DERIVATIVE (PROFILE) 

x_prof = X; f_prof = extrapolF;
% 1D direct FFT calculation
y = fft(f_prof);

% wave-numbers evaluation (saved in the field fk() )
% df is the step of the wave-number
dx = x_prof(2)-x_prof(1);
df = 1/(dx*(pointsnextr-1));
R=rem(pointsnextr,2);
if R==0
       fk(1:pointsnextr/2+1) = 0:df:(pointsnextr/2)*df;
       fk_pom = -fk(2:pointsnextr/2+1);
       fk(pointsnextr/2+1:pointsnextr) = fk_pom(:,[length(fk_pom):-1:1]);
else
       fk(1:round(pointsnextr/2))=0:df:floor(pointsnextr/2)*df;
       fk_pom = -fk(2:round(pointsnextr/2));
       fk(round(pointsnextr/2)+1:pointsnextr) = fk_pom(:,[length(fk_pom):-1:1]);
end;

% L-NORMS EVALUATION FOR THE REGULARIZED HORIZONTAL DERIVATIVE
% initialisation of a help array - prev() -it will contain previous reconstructed field (for previous alpha)
for ji=1:pointsnextr
   prev(ji) = 0;
end;

% input of regular. parameter limits in a window
 prompti = {'start value of regular. parameter (e.g. 1E-5)','end value of regular. parameter (e.g. 1E+20)'};
 titlei = 'norm (x-derivative)';
 linesi = 1; defi = {'1E-5','1E+20'};
 answer = inputdlg(prompti,titlei,linesi,defi);
 opergstr1 = answer(1); alphabegin = str2double(opergstr1);
 opergstr2 = answer(2); alphaend = str2double(opergstr2);
 
% begin of the main cycle - for changing of alpha in a geometrical sequence
% setting the starting value of alpha,  kj is the order number for the increasing alpha
alpha = alphabegin; kj = 1; alpha
% the final value of alpha is set as the limit of the main cycle
while alpha<alphaend
   % evaluation of the regularized horizontal derivative in spectral domain by means of the Tikhonov filter
   for ji=1:pointsnextr
   yt(ji) = y(ji) *sqrt(-1)*(fk(ji)*2*pi)/(1+alpha*fk(ji)*fk(ji)*tanh(fk(ji)*2*pi)*tanh(fk(ji)*2*pi));
   end;
   % inverse DFT
   B = real(ifft(yt));
   %L(p)-norms evaluation
   arr_alpha(kj) = alpha;
   norm_function0_5(kj) = norm(B-prev,0.5);norm_function0_7(kj) = norm(B-prev,0.7);
   norm_function1(kj) = norm(B-prev,1); norm_function2(kj) = norm(B-prev,2); norm_functionC(kj) = norm(B-prev,inf);
   prev = B;
alpha = 1.1*alpha; kj = kj + 1;
end; 
kj = kj - 2; alpha
% end of the main cycle - for changing of alpha in a geometrical sequence

% finding of local minimum of the C norm function for the x-derivative
% the first value of the norm function can not be evaluated correctly, it is taken equal to the second value
norm_functionC(1) = norm_functionC(2);
ndata = kj - 1; position = 0; optalphax = 0; optnormx = 0;
for ji=(ndata-1): -1:3
     if (norm_functionC(ji)<norm_functionC(ji-1)) & (norm_functionC(ji+1)>norm_functionC(ji))
     position = ji; optalphax = arr_alpha(ji); optnormx = norm_functionC(ji);
     end;
end;

% plotting the L norms for the x-derivative
if optalphax ~= 0
   xnorm_2D=figure('Name','L norms for the x-derivative','NumberTitle','off');
   loglog(arr_alpha,norm_function0_5,arr_alpha,norm_function0_7,arr_alpha,norm_function1,arr_alpha,norm_function2,arr_alpha,norm_functionC,optalphax,optnormx,...
           '--rs','LineWidth',1,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',10)
   xlabel('log10(alfa) []','FontSize',10); ylabel('log10{Lnorm[reconstr(j)-reconst(j-1)]} [nT (or) mGal]','FontSize',10);
   legend('L_0_5 norm','L_0_7 norm','L_1 norm','L_2 norm','C = L_i_n_f norm','Location','South');
   title({'L norms for the x-derivative';'Press ''Z''key to enable picking mode or any key to disable picking mode'});
else
   xnorm_2D=figure('Name','L norms for the x-derivative','NumberTitle','off');
   loglog(arr_alpha,norm_function0_5,arr_alpha,norm_function0_7,arr_alpha,norm_function1,arr_alpha,norm_function2,arr_alpha,norm_functionC,optalphax,optnormx,...
           '--rs','LineWidth',1,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',10)
   xlabel('log10(alfa) []','FontSize',10); ylabel('log10{Lnorm[reconstr(j)-reconst(j-1)]} [nT (or) mGal]','FontSize',10);
   legend('L_0_5 norm','L_0_7 norm','L_1 norm','L_2 norm','C = L_i_n_f norm','Location','South');
   title({'L norms for the x-derivative, NO LOCAL MINIMUM FOUND';'Press ''Z''key to enable picking mode or any key to disable picking mode'});
end;

out_pos=get(xnorm_2D,'OuterPosition');scr=get(0,'ScreenSize');pos=get(xnorm_2D,'Position');
set(xnorm_2D,'Position',[scr(3)/2-out_pos(3) scr(4)/4 pos(3) pos(4)])

set(xnorm_2D,'keypressfcn',@normpick_2D)
x_deriv_2D(optalphax,optalphax);


% saving the C-norm-function
  for ji=1:ndata
       savearr(ji,1) = arr_alpha(ji);
       savearr(ji,2) = norm_functionC(ji);
  end;  
     part1='Cnorm_xderiv_';part2=fnameprofile; nametogether=strcat(part1,part2);
     cd(pname_prof);  
     savefile = fopen(nametogether, 'w');
     fprintf(savefile,'%s %s','alpha','Lnorm(inf)');
     fprintf(savefile, '\n%e %e', savearr');
     fclose(savefile);
     cd(c_dir_prof);
if optalphax == arr_alpha(ndata)
   optalphax = 0;
end;

function z_derivat_norm_2D_Callback(src,eventdata)
global X F m optalphaz extrapolF numextrPoints pointsnextr fnameprofile pname_prof c_dir_prof

%initial check
if isempty(extrapolF)
    helpdlg('Load profile data first','Wrong selection');
    return
else
end

%%%%%%% EVALUATION OF THE NORM FOR THE REGULARIZED Z-DERIVATIVE (PROFILE) 

x_prof = X; f_prof = extrapolF;
% 1D direct FFT calculation
y = fft(f_prof);

% wave-numbers evaluation (saved in the field fk() )
% df is the step of the wave-number
dx = x_prof(2)-x_prof(1);
df = 1/(dx*(pointsnextr-1));
R=rem(pointsnextr,2);
if R==0
       fk(1:pointsnextr/2+1) = 0:df:(pointsnextr/2)*df;
       fk_pom = -fk(2:pointsnextr/2+1);
       fk(pointsnextr/2+1:pointsnextr) = fk_pom(:,[length(fk_pom):-1:1]);
else
       fk(1:round(pointsnextr/2))=0:df:floor(pointsnextr/2)*df;
       fk_pom = -fk(2:round(pointsnextr/2));
       fk(round(pointsnextr/2)+1:pointsnextr) = fk_pom(:,[length(fk_pom):-1:1]);
end;

% L-NORMS EVALUATION FOR THE REGULARIZED HORIZONTAL DERIVATIVE
% initialisation of a help array - prev() -it will contain previous reconstructed field (for previous alpha)
for ji=1:pointsnextr
   prev(ji) = 0;
end;

% input of regular. parameter limits in a window
 prompti = {'start value of regular. parameter (e.g. 1E-5)','end value of regular. parameter (e.g. 1E+20)'};
 titlei = 'norm (z-derivative))';
 linesi = 1; defi = {'1E-5','1E+20'};
 answer = inputdlg(prompti,titlei,linesi,defi);
 opergstr1 = answer(1); alphabegin = str2double(opergstr1);
 opergstr2 = answer(2); alphaend = str2double(opergstr2);

% begin of the main cycle - for changing of alpha in a geometrical sequence
% setting the starting value of alpha,  kj is the order number for the increasing alpha
alpha = alphabegin; kj = 1; alpha
% the final value of alpha is set as the limit of the main cycle
while alpha<alphaend
   % evaluation of the regularized vertical derivative in spectral domain by means of the Tikhonov filter
   for ji=1:pointsnextr
   yt(ji) = y(ji) *(abs(fk(ji))*2*pi)/(1+alpha*fk(ji)*fk(ji)*tanh(fk(ji)*2*pi)*tanh(fk(ji)*2*pi));
   end;
   % inverse DFT
   B = real(ifft(yt));
   %L(p)-norms evaluation
   arr_alpha(kj) = alpha;
   norm_function0_5(kj) = norm(B-prev,0.5);
   norm_function0_7(kj) = norm(B-prev,0.7);
   norm_function1(kj) = norm(B-prev,1); 
   norm_function2(kj) = norm(B-prev,2); 
   norm_functionC(kj) = norm(B-prev,inf);
   prev = B;
alpha = 1.1*alpha; kj = kj + 1;
end; 
kj = kj - 2; alpha
% end of the main cycle - for changing of alpha in a geometrical sequence

% finding of local minimum of the C norm function for the x-derivative
% the first value of the norm function can not be evaluated correctly, it is taken equal to the second value
norm_functionC(1) = norm_functionC(2);
ndata = kj - 1; position = 0; optalphaz = 0; optnomrz = 0;
for ji=(ndata-1): -1:3
     if (norm_functionC(ji)<norm_functionC(ji-1)) & (norm_functionC(ji+1)>norm_functionC(ji))
     position = ji; optalphaz = arr_alpha(ji); optnormz = norm_functionC(ji);
     end;
end;

% plotting the L norms for the z-derivative
if optalphaz ~= 0
   znorm_2D=figure('Name','L norms for the z-derivative','NumberTitle','off');
   loglog(arr_alpha,norm_function0_5,arr_alpha,norm_function0_7,arr_alpha,norm_function1,arr_alpha,norm_function2,arr_alpha,norm_functionC,optalphaz,optnormz,...
           '--rs','LineWidth',1,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',10)
   xlabel('log10(alfa) []','FontSize',10); ylabel('log10{Lnorm[reconstr(j)-reconst(j-1)]} [nT (or) mGal]','FontSize',10);
   legend('L_0_5 norm','L_0_7 norm','L_1 norm','L_2 norm','C = L_i_n_f norm','Location','South');
   title({'L norms for the z-derivative';'Press ''Z''key to enable picking mode or any key to disable picking mode'});
else
   znorm_2D=figure('Name','L norms for the z-derivative','NumberTitle','off');
   loglog(arr_alpha,norm_function0_5,arr_alpha,norm_function0_7,arr_alpha,norm_function1,arr_alpha,norm_function2,arr_alpha,norm_functionC,optalphaz,optnormz,...
           '--rs','LineWidth',1,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',10)
   xlabel('log10(alfa) []','FontSize',10); ylabel('log10{Lnorm[reconstr(j)-reconst(j-1)]} [nT (or) mGal]','FontSize',10);
   legend('L_0_5 norm','L_0_7 norm','L_1 norm','L_2 norm','C = L_i_n_f norm','Location','South');
   title({'L norms for the z-derivative, NO LOCAL MINIMUM FOUND';'Press ''Z''key to enable picking mode or any key to disable picking mode'});
end;

out_pos=get(znorm_2D,'OuterPosition');scr=get(0,'ScreenSize');pos=get(znorm_2D,'Position');
set(znorm_2D,'Position',[scr(3)/2-out_pos(3) (scr(4)/4)-20 pos(3) pos(4)])
set(znorm_2D,'keypressfcn',@normpick_2D)
z_deriv_2D(optalphaz,optalphaz);
% saving the C-norm-function
  for ji=1:ndata
       savearr(ji,1) = arr_alpha(ji);
       savearr(ji,2) = norm_functionC(ji);
  end;
  part1='Cnorm_zderiv_';part2=fnameprofile; nametogether=strcat(part1,part2);
 cd(pname_prof);  
 savefile = fopen(nametogether, 'w');
 fprintf(savefile,'%s %s','alpha','Lnorm(inf)');
 fprintf(savefile, '\n%e %e', savearr');
 fclose(savefile);
 cd(c_dir_prof);
  
if optalphaz == arr_alpha(ndata)
   optalphaz = 0;
end;

% --------------------------------------------------------------------
function x_derivat_calc_2D_Callback(src,eventdata)
global optalphax extrapolF
%initial check
if isempty(extrapolF)
    helpdlg('Load profile data first','Wrong selection');
    return
else
end

%%%%%% FINAL X-DERIVATIVE (PROFILE) EVALUATION 
% input of regular. parameter limits in a window
 prompti = {'optimum value of regular. parameter (0 for non-regular.)'};
 titlei = 'set parameter';
 linesi = 1; defi = {num2str(optalphax)};
 answer = inputdlg(prompti,titlei,linesi,defi);
 opergstr1 = answer(1); optalphax = str2double(opergstr1);
 x_deriv_2D(optalphax);

function x_deriv_2D(varargin)
global X m extrapolF numextrPoints pointsnextr fnameprofile fig_xder_2D xder_2D optalphax prof_regdx pname_prof c_dir_prof

%independent function for the calculation core
 inp=length(varargin);

%input arguments
if inp==1;
   optalphax=varargin{1}
elseif inp==2;
   optalphax=varargin{1};pick_alpha=varargin{2};optalphax=pick_alpha;
end


% regularized x-derivative calculation (from profile data)
x_prof = X; f_prof = extrapolF;
% 1D direct FFT calculation
y = fft(f_prof);

% wave-numbers evaluation (saved in the field fk() )
% df is the step of the wave-number
dx = x_prof(2)-x_prof(1);
df = 1/(dx*(pointsnextr-1));
R=rem(pointsnextr,2);
if R==0
       fk(1:pointsnextr/2+1) = 0:df:(pointsnextr/2)*df;
       fk_pom = -fk(2:pointsnextr/2+1);
       fk(pointsnextr/2+1:pointsnextr) = fk_pom(:,[length(fk_pom):-1:1]);
else
       fk(1:round(pointsnextr/2))=0:df:floor(pointsnextr/2)*df;
       fk_pom = -fk(2:round(pointsnextr/2));
       fk(round(pointsnextr/2)+1:pointsnextr) = fk_pom(:,[length(fk_pom):-1:1]);
end;
 
% calculation of the final x-derived field (for the optimum or selected value of alpha)
   for ji=1:pointsnextr
   yt(ji) = y(ji) *sqrt(-1)*(fk(ji)*2*pi)/(1+optalphax*fk(ji)*fk(ji)*tanh(fk(ji)*2*pi)*tanh(fk(ji)*2*pi));
   end;
   % inverse DFT
   prof_regdx2 = real(ifft(yt));

%extracting the original length of the profile
   prof_regdx = prof_regdx2(numextrPoints+1:pointsnextr-numextrPoints);

% plotting the graph of the final x-derived data (when the optimum alpha is not equal to the last used alpha)

if inp==2
   if pick_alpha>99999
      title_var = '%.3e';
   else
      title_var = '%.3f';
   end
   
   if ishandle(findobj('Name','regularized x-derivative'))
        title_solid=('output - regularized x-derivative, picked alpha value:');
        title_format=strcat(title_solid,title_var);title_str=sprintf(title_format, pick_alpha);
        set(fig_xder_2D,'Xdata',x_prof,'Ydata',prof_regdx)
        if pick_alpha==0
           title_str=('output - NON-regular. x-derivative (alpha = 0)');
        end
        title(get(fig_xder_2D,'Parent'),title_str)
   else
        title_solid=('output - regularized x-derivative, picked alpha value:');
        title_format=strcat(title_solid,title_var);title_str=sprintf(title_format, pick_alpha);
        xder_2D=figure('Name','regularized x-derivative','NumberTitle','off');
        fig_xder_2D=plot(x_prof,prof_regdx); xlabel('x [dist.units]','FontSize',10); ylabel('x-derived function [nT/m (or) mGal/m]','FontSize',10);
        legend('x-derived');
        if pick_alpha==0
           title_str=('output - NON-regular. x-derivative (alpha = 0)');
        end
        title(title_str);
   end   
       
else
    if optalphax ~= 0
       xder_2D=figure('Name','regularized x-derivative','NumberTitle','off');
       fig_xder_2D=plot(x_prof,prof_regdx); xlabel('x [dist.units]','FontSize',10); ylabel('x-derived function [nT/m (or) mGal/m]','FontSize',10);
       legend('x-derived');title('output - regularized x-derivative (select z-derivative calc.)');
    else
       xder_2D=figure('Name','NON-regular. x-derivative','NumberTitle','off');
       fig_xder_2D=plot(x_prof,prof_regdx); xlabel('x [dist.units]','FontSize',10); ylabel('x-derived function [nT/m (or) mGal/m]','FontSize',10);
       legend('x-derived');title('output - NON-regular. x-derivative (alpha = 0) (select z-derivative calc.)');
    end;
end

out_pos=get(xder_2D,'OuterPosition');scr=get(0,'ScreenSize');pos=get(xder_2D,'Position');
set(xder_2D,'Position',[scr(3)/2 scr(4)/4 pos(3) pos(4)])

% saving the final x-derived profile data (non-regularized for alpha=0, regularized for alpha<>0)
   for ji=1:m
       savearr(ji,1) = x_prof(ji);
       savearr(ji,2) = prof_regdx(ji);
   end;
if optalphax ~=0
     part1='x-regderiv_';part2=fnameprofile; nametogether=strcat(part1,part2);
     cd(pname_prof);  
 savefile = fopen(nametogether, 'w');
 fprintf(savefile,'%s %s','X','regDx');
 fprintf(savefile, '\n%e %e', savearr');
 fclose(savefile);
     cd(c_dir_prof);
else
     part1='x-nonregder_';part2=fnameprofile; nametogether=strcat(part1,part2);
     cd(pname_prof);  
 savefile = fopen(nametogether, 'w');
 fprintf(savefile,'%s %s','X','nonregDx');
 fprintf(savefile, '\n%e %e', savearr');
 fclose(savefile);     cd(c_dir_prof);
end;

% --------------------------------------------------------------------
function z_derivat_calc_2D_Callback(src,eventdata)
global optalphaz extrapolF
%initial check
if isempty(extrapolF)
    helpdlg('Load profile data first','Wrong selection');
    return
else
end

%%%%%% FINAL Z-DERIVATIVE (PROFILE) EVALUATION 

% input of regular. parameter limits in a window
 prompti = {'optimum value of regular. parameter (0 for non-regular.)'};
 titlei = 'set parameter';
 linesi = 1; defi = {num2str(optalphaz)};
 answer = inputdlg(prompti,titlei,linesi,defi);
 opergstr1 = answer(1); optalphaz = str2double(opergstr1);
 
 z_deriv_2D(optalphaz);
 
function z_deriv_2D(varargin)
global X m extrapolF numextrPoints pointsnextr fnameprofile fig_zder_2D zder_2D optalphaz prof_regdz pname_prof c_dir_prof
%independent function for the calculation core
 inp=length(varargin);

%input arguments
if inp==1;
   optalphaz=varargin{1};
elseif inp==2;
   optalphaz=varargin{1};pick_alpha=varargin{2};optalphaz=pick_alpha;
end

     
x_prof = X; f_prof = extrapolF;
% 1D direct FFT calculation
y = fft(f_prof);

% wave-numbers evaluation (saved in the field fk() )
% df is the step of the wave-number
dx = x_prof(2)-x_prof(1);
df = 1/(dx*(pointsnextr-1));
R=rem(pointsnextr,2);
if R==0
       fk(1:pointsnextr/2+1) = 0:df:(pointsnextr/2)*df;
       fk_pom = -fk(2:pointsnextr/2+1);
       fk(pointsnextr/2+1:pointsnextr) = fk_pom(:,[length(fk_pom):-1:1]);
else
       fk(1:round(pointsnextr/2))=0:df:floor(pointsnextr/2)*df;
       fk_pom = -fk(2:round(pointsnextr/2));
       fk(round(pointsnextr/2)+1:pointsnextr) = fk_pom(:,[length(fk_pom):-1:1]);
end;

% calculation of the final z-derived field (for the optimum or selected value of alpha)
   for ji=1:pointsnextr
   yt(ji) = y(ji) *(abs(fk(ji))*2*pi)/(1+optalphaz*fk(ji)*fk(ji)*tanh(fk(ji)*2*pi)*tanh(fk(ji)*2*pi));
   end;
   % inverse DFT
   prof_regdz2 = real(ifft(yt));
   
%extracting the original length of the profile
   prof_regdz = prof_regdz2(numextrPoints+1:pointsnextr-numextrPoints);
   
% plotting the graph of the final z-derived data (when the optimum alpha is not equal to the last used alpha)
if inp==2
   if pick_alpha>99999
      title_var = '%.3e';
   else
      title_var = '%.3f';
   end
   
   if ishandle(findobj('Name','regularized z-derivative'))
        title_solid1=('output - regularized z-derivative, picked alpha value:');
        title_format=strcat(title_solid1,title_var);title_str=sprintf(title_format, pick_alpha);
        set(fig_zder_2D,'Xdata',x_prof,'Ydata',prof_regdz)
        if pick_alpha==0
           title_str=('output - NON-regular. z-derivative (alpha = 0)');
        end
        title(get(fig_zder_2D,'Parent'),title_str)
   else
        title_solid1=('output - regularized z-derivative, picked alpha value:');
        title_format=strcat(title_solid1,title_var);title_str=sprintf(title_format, pick_alpha);
        zder_2D=figure('Name','regularized z-derivative','NumberTitle','off');
        fig_zder_2D=plot(x_prof,prof_regdz); xlabel('x [dist.units]','FontSize',10); ylabel('z-derived function [nT/m (or) mGal/m]','FontSize',10);
        legend('z-derived');
        if pick_alpha==0
           title_str=('output - NON-regular. z-derivative (alpha = 0)');
        end
        title(title_str);
   end   
       
else
    if optalphaz ~= 0
       zder_2D=figure('Name','regularized z-derivative','NumberTitle','off')
       fig_zder_2D=plot(x_prof,prof_regdz); xlabel('x [dist.units]','FontSize',10); ylabel('z-derived function [nT/m (or) mGal/m]','FontSize',10);
       legend('z-derived');title('output - regularized z-derivative ');
    else
       zder_2D=figure('Name','NON-regular. z-derivative','NumberTitle','off')
       fig_zder_2D=plot(x_prof,prof_regdz); xlabel('x [dist.units]','FontSize',10); ylabel('z-derived function [nT/m (or) mGal/m]','FontSize',10);
       legend('z-derived');title('output - NON-regular. z-derivative (alpha = 0)');
    end;
end

out_pos=get(zder_2D,'OuterPosition');scr=get(0,'ScreenSize');pos=get(zder_2D,'Position');
set(zder_2D,'Position',[scr(3)/2 (scr(4)/4)-20 pos(3) pos(4)])


% saving the final z-derived profile data (non-regularized for alpha=0,regularized for alpha<>0)
   for ji=1:m
       savearr(ji,1) = x_prof(ji);
       savearr(ji,2) = prof_regdz(ji);
   end;
if optalphaz ~=0
     part1='z-regderiv_';part2=fnameprofile; nametogether=strcat(part1,part2);
     cd(pname_prof);  
 savefile = fopen(nametogether, 'w');
 fprintf(savefile,'%s %s','X','regDz');
 fprintf(savefile, '\n%e %e', savearr');
 fclose(savefile);     cd(c_dir_prof);
else
     part1='z-nonregder_';part2=fnameprofile; nametogether=strcat(part1,part2);
     cd(pname_prof);  
 savefile = fopen(nametogether, 'w');
 fprintf(savefile,'%s %s','X','nonregDz');
 fprintf(savefile, '\n%e %e', savearr');
 fclose(savefile);     cd(c_dir_prof);
end;    


%2. GRID DATA~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
function Open_grid_data_Callback(src,eventdata)
global columnsn rowsm minx maxx miny maxy field extrapolfield numextrRows numextrCols rowsmextr columnsnextr
global fnamegrid xax yax c_dir_grid pname_grid

rowsm = 0; columnsn = 0; minx = 0; maxx= 0; miny = 0; maxy= 0; minf = 0; maxf= 0;
[fnamegrid,pname_grid] = uigetfile('*.grd','input of data (Surfer ASCII *.grid file)');
c_dir_grid=pwd;cd (pname_grid)
fid = fopen(fnamegrid,'r');
cd (c_dir_grid)
% reading of the GS ASCII grid header (rows, columns, minx, maxx, miny, maxy, minf, maxf)
tline = fgetl(fid); row1 = tline;
tline = fgetl(fid); row2 = tline; [token,rem] = strtok(row2);
rowsm = str2double(token); columnsn = str2double(rem);
tline = fgetl(fid); row3 = tline; [token,rem] = strtok(row3);
minx = str2double(token); maxx = str2double(rem);
tline = fgetl(fid); row4 = tline; [token,rem] = strtok(row4);
miny = str2double(token); maxy = str2double(rem);
tline = fgetl(fid); row5 = tline; [token,rem] = strtok(row5);
minf = str2double(token); maxf = str2double(rem);
% reading of the main field of the grid - into the matrix A
[A,count] = fscanf(fid,'%f',[rowsm,columnsn]);
status = fclose(fid);
field = A';
% size of the readed matrix: columnsn x rowsm
[columnsn,rowsm] = size(A);
s = [' size of readed grid: ' int2str(rowsm) ' rows x ' int2str(columnsn) ' columns                                    (select next items from the "Grid data regul.deriv." menu)'];

%field axes set up
[mg,ng]=size(field);
xax=(minx:(maxx-minx)/(ng-1):maxx);
yax=(miny:(maxy-miny)/(mg-1):maxy);

%field extrapolation - 15% each edge (to exclude the edge effects during FFT)
[extrapolfield,numextrRows,numextrCols] = cosxp2(field,0.15);% 0.15=15% of extrapolation
%new dimensions of the extrapolated field
[rowsmextr, columnsnextr] = size(extrapolfield);
%Display data and show info
 Display_grid_data_Callback
 msgbox(s,'message');


function Display_grid_data_Callback(src,eventdata)
global field extrapolfield minx maxx miny maxy yax xax

%initial check
if isempty(field)
    helpdlg('Load grid data first','Wrong selection');
    return
else
end

% plotting the contour map of the readed input grid data
orig_fig_g=figure('Name','original data (3D)','NumberTitle','off');
pcolor(xax,yax,field); colorbar; legend('original data');shading interp;
xlabel ('x[dist.units]','FontSize',10); ylabel('y[dist.units]','FontSize',10);
title('input - original imported grid data   (select next items from "grid" menu)   [mGal or nT]');
hold on;contour(xax,yax,field,'k');hold off;
colormap(gca(orig_fig_g),colscale)
set(orig_fig_g,'WindowButtonMotionFcn',@cursorcoordinate)



% --------------------------------------------------------------------
function x_derivat_norm_3D_Callback(src,eventdata)
global columnsn columnsnextr rowsm rowsmextr minx maxx miny maxy field extrapolfield optalphaxg
global fnamegrid pname_grid c_dir_grid
%initial check
if isempty(field)
    helpdlg('Load grid data first','Wrong selection');
    return
else
end

%%%%%%%% EVALUATION OF THE NORM FOR THE REGULARIZED X-DERIVATIVE (GRID) 
% 2D direct FFT calculation
y = fft2(extrapolfield);

% input of regular. parameter limits in a window
 inform = 'evaluation of the norm (x-derivative)'; inform
 prompti = {'start value of regular. parameter (e.g. 1E0)','end value of regular. parameter (e.g. 1E+20)'};
 titlei = 'norm (x-derivative)';
 linesi = 1; defi = {'1E0','1E+20'};
 answer = inputdlg(prompti,titlei,linesi,defi);
 opergstr1 = answer(1); alphabeging = str2double(opergstr1);
 opergstr2 = answer(2); alphaendg = str2double(opergstr2);

% wave-numbers evaluation (saved in the fields fkx() and fky()  )
% dfx and dfy  are the steps of the wave-numbers
dx = abs(maxx-minx)/(columnsn-1); dy = abs(maxy-miny)/(rowsm-1);
dfx = 1/(dx*(columnsnextr-1)); dfy = 1/(dy*(rowsmextr-1));
R=rem(columnsnextr,2);
if R==0
       fkx(1:columnsnextr/2+1) = 0:dfx:(columnsnextr/2)*dfx;
       fkx_pom = -fkx(2:columnsnextr/2+1);
       fkx(columnsnextr/2+1:columnsnextr) = fkx_pom(:,[length(fkx_pom):-1:1]);
else
       fkx(1:round(columnsnextr/2))=0:dfx:floor(columnsnextr/2)*dfx;
       fkx_pom = -fkx(2:round(columnsnextr/2));
       fkx(round(columnsnextr/2)+1:columnsnextr) = fkx_pom(:,[length(fkx_pom):-1:1]);
end;
R=rem(rowsmextr,2);
if R==0
       fky(1:rowsmextr/2+1) = 0:dfy:(rowsmextr/2)*dfy;
       fky_pom = -fky(2:rowsmextr/2+1);
       fky(rowsmextr/2+1:rowsmextr) = fky_pom(:,[length(fky_pom):-1:1]);
else
       fky(1:round(rowsmextr/2))=0:dfy:floor(rowsmextr/2)*dfy;
       fky_pom = -fky(2:round(rowsmextr/2));
       fky(round(rowsmextr/2)+1:rowsmextr) = fky_pom(:,[length(fky_pom):-1:1]);
end;  
  
%MAIN CYCLE OF EVALUATION OF THE NORM FOR THE REGULARIZED X-DERIVATIVE (GRID)
% initialisation of a help matrix - prevji() -it will contain previous reconstructed field (for previous alpha)
prevji = zeros(rowsmextr,columnsnextr);
% begin of the main cycle - for changing of alpha in a geometrical sequence
% setting the starting value of alpha,  kj is the order number for the increasing alpha
alpha = alphabeging; kj = 1; alpha
% the final value of alpha is set as the limit of the main cycle
while alpha<alphaendg
%alpha
   % evaluation of the x-horizontal derivative in spectral domain by means of the Tikhonov filter application
   for j=1:rowsmextr
       for k=1:columnsnextr
        yt(j,k) = y(j,k)*sqrt(-1)*fkx(k)*2*pi/(1+(alpha/(4*pi*pi))*(fkx(k)*fkx(k)+fky(j)*fky(j))*tanh(2*pi*sqrt(fkx(k)*fkx(k)+fky(j)*fky(j)))*tanh(2*pi*sqrt(fkx(k)*fkx(k)+fky(j)*fky(j))));
	   end;
   end;
   % inverse DFT
   B = real(ifft2(yt));
   
 % Cnorm-function calculation
   maxdiffer = 0;
   for j=2:rowsmextr-1
     for k=2:columnsnextr-1
       if abs(B(j,k)-prevji(j,k))>maxdiffer
          maxdiffer = abs(B(j,k)-prevji(j,k)); 
       end;
     end;
   end;
   arr_alpha(kj) = alpha;
   norm_function(kj) = maxdiffer;
 %L(p)-norms evaluation
   norm_function1(kj) = norm(B-prevji,1); norm_function2(kj) = norm(B-prevji,2);
   norm_functionC(kj) = norm(B-prevji,inf); norm_function_Fro(kj) = norm(B-prevji,'fro');
   % filling of the help matrix prevji() with the last actual solution (for last actual alpha)
   prevji = B;
% increasing of alpha (in a geometrical sequence, with a quocient of 1.1)
alpha = 1.1*alpha; kj = kj + 1;
end; 
kj = kj - 2; alpha
% end of the main cycle - for changing of alpha in a geometrical sequence

% finding of local minimum of the C norm function for the x-derivative
% the first value of the norm function can not be evaluated correctly, it is taken equal to the second value
norm_function(1) = norm_function(2);
ndata = kj - 1; position = 0; optalphaxg = 0; optnormx = 0;
for ji=(ndata-1): -1:3
     if (norm_function(ji)<norm_function(ji-1)) & (norm_function(ji+1)>norm_function(ji))
     position = ji; optalphaxg = arr_alpha(ji); optnormx = norm_function(ji);
     end;
end;
% plotting the L-norms for the x-derivative
if optalphaxg ~= 0
   xnorm_3D=figure('Name','L norms for the x-derivative(3D)','NumberTitle','off');
   loglog(arr_alpha,norm_function1,arr_alpha,norm_function2,arr_alpha,norm_function,optalphaxg,optnormx,...
           '--rs','LineWidth',1,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',10)
   xlabel('log10(alfa) []','FontSize',10); ylabel('log10Cnorm{reconstr(j)]-reconst(j-1)} [nT (or) mGal]','FontSize',10);
   legend('L_1 norm','L_2 norm','C = L_i_n_f norm','Location','South');
   title({'L norms for the x-derivative';'Press ''Z''key to enable picking mode or any key to disable picking mode'});
else
   xnorm_3D=figure('Name','L norms for the x-derivative(3D)','NumberTitle','off');
   loglog(arr_alpha,norm_function1,arr_alpha,norm_function2,arr_alpha,norm_function,optalphaxg,optnormx,...
           '--rs','LineWidth',1,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',10)
   xlabel('log10(alfa) []','FontSize',10); ylabel('log10Cnorm{reconstr(j)]-reconst(j-1)} [nT (or) mGal]','FontSize',10);
   legend('L_1 norm','L_2 norm','C = L_i_n_f norm','Location','South');
   title({'L norms for the x-derivative, NO LOCAL MINIMUM FOUND';'Press ''Z''key to enable picking mode or any key to disable picking mode'});
end;

out_pos=get(xnorm_3D,'OuterPosition');scr=get(0,'ScreenSize');pos=get(xnorm_3D,'Position');
set(xnorm_3D,'Position',[scr(3)/2-out_pos(3) (scr(4)/4)+20 pos(3) pos(4)])
set(xnorm_3D,'keypressfcn',@normpick_2D)

x_deriv_3D(optalphaxg,optalphaxg);

% saving the C-norm-function
  for ji=1:ndata
       savearr(ji,1) = log10(arr_alpha(ji));
       savearr(ji,2) = log10(norm_functionC(ji));
  end;
  part1='Cnorm_xderiv_';part2=strrep(fnamegrid,'.grd','');part3= '.dat'; nametogether=strcat(part1,part2,part3);
  cd (pname_grid)
 savefile = fopen(nametogether, 'w');
 fprintf(savefile,'%s %s','alpha','Lnorm(inf)');
 fprintf(savefile, '\n%e %e', savearr');
 fclose(savefile);
 cd (c_dir_grid)
  
function y_derivat_norm_3D_Callback(src,eventdata)
global columnsn columnsnextr rowsm rowsmextr minx maxx miny maxy field extrapolfield optalphayg
global fnamegrid pname_grid c_dir_grid
%initial check
if isempty(field)
    helpdlg('Load grid data first','Wrong selection');
    return
else
end

%%%%%%%% EVALUATION OF THE NORM FOR THE REGULARIZED Y-DERIVATIVE (GRID)
% 2D direct FFT calculation
y = fft2(extrapolfield);

% input of regular. parameter limits in a window
 inform = 'evaluation of the norm (y-derivative)'; inform
 prompti = {'start value of regular. parameter (e.g. 1E0)','end value of regular. parameter (e.g. 1E+20)'};
 titlei = 'norm (y-derivative)';
 linesi = 1; defi = {'1E0','1E+20'};
 answer = inputdlg(prompti,titlei,linesi,defi);
 opergstr1 = answer(1); alphabeging2 = str2double(opergstr1);
 opergstr2 = answer(2); alphaendg2 = str2double(opergstr2);

% wave-numbers evaluation (saved in the fields fkx() and fky()  )
% dfx and dfy  are the steps of the wave-numbers
dx = abs(maxx-minx)/(columnsn-1); dy = abs(maxy-miny)/(rowsm-1);
dfx = 1/(dx*(columnsnextr-1)); dfy = 1/(dy*(rowsmextr-1));
R=rem(columnsnextr,2);
if R==0
       fkx(1:columnsnextr/2+1) = 0:dfx:(columnsnextr/2)*dfx;
       fkx_pom = -fkx(2:columnsnextr/2+1);
       fkx(columnsnextr/2+1:columnsnextr) = fkx_pom(:,[length(fkx_pom):-1:1]);
else
       fkx(1:round(columnsnextr/2))=0:dfx:floor(columnsnextr/2)*dfx;
       fkx_pom = -fkx(2:round(columnsnextr/2));
       fkx(round(columnsnextr/2)+1:columnsnextr) = fkx_pom(:,[length(fkx_pom):-1:1]);
end;
R=rem(rowsmextr,2);
if R==0
       fky(1:rowsmextr/2+1) = 0:dfy:(rowsmextr/2)*dfy;
       fky_pom = -fky(2:rowsmextr/2+1);
       fky(rowsmextr/2+1:rowsmextr) = fky_pom(:,[length(fky_pom):-1:1]);
else
       fky(1:round(rowsmextr/2))=0:dfy:floor(rowsmextr/2)*dfy;
       fky_pom = -fky(2:round(rowsmextr/2));
       fky(round(rowsmextr/2)+1:rowsmextr) = fky_pom(:,[length(fky_pom):-1:1]);
end;  
  
%MAIN CYCLE OF EVALUATION OF THE NORM FOR THE REGULARIZED Y-DERIVATIVE (GRID)
% initialisation of a help matrix - prevji() -it will contain previous reconstructed field (for previous alpha)
prevji = zeros(rowsmextr,columnsnextr);
% begin of the main cycle - for changing of alpha in a geometrical sequence
% setting the starting value of alpha,  kj is the order number for the increasing alpha
alpha = alphabeging2; kj = 1; alpha
% the final value of alpha is set as the limit of the main cycle
while alpha<alphaendg2
%alpha
   % evaluation of the y-horizontal derivative in spectral domain by means of the Tikhonov filter application
   for j=1:rowsmextr
       for k=1:columnsnextr
	    yt(j,k) = y(j,k)*sqrt(-1)*fky(j)*2*pi/(1+(alpha/(4*pi*pi))*(fkx(k)*fkx(k)+fky(j)*fky(j))*tanh(2*pi*sqrt(fkx(k)*fkx(k)+fky(j)*fky(j)))*tanh(2*pi*sqrt(fkx(k)*fkx(k)+fky(j)*fky(j))));
	   end;
   end;
   % inverse DFT
   B = real(ifft2(yt));

% Cnorm-function calculation
   maxdiffer = 0;
   for j=2:rowsmextr-1
     for k=2:columnsnextr-1
       if abs(B(j,k)-prevji(j,k))>maxdiffer
          maxdiffer = abs(B(j,k)-prevji(j,k)); 
       end;
     end;
   end;
   arr_alpha(kj) = alpha;
   norm_function(kj) = maxdiffer;
   %L(p)-norms evaluation
   norm_function1(kj) = norm(B-prevji,1); norm_function2(kj) = norm(B-prevji,2);
   norm_functionC(kj) = norm(B-prevji,inf); norm_function_Fro(kj) = norm(B-prevji,'fro');
   % filling of the help matrix prevji() with the last actual solution (for last actual alpha)
   prevji = B;
% increasing of alpha (in a geometrical sequence, with a quocient of 1.1)
alpha = 1.1*alpha; kj = kj + 1;
end; 
kj = kj - 2; alpha
% end of the main cycle - for changing of alpha in a geometrical sequence

% finding of local minimum of the C norm function for the y-derivative
% the first value of the norm function can not be evaluated correctly, it is taken equal to the second value
norm_function(1) = norm_function(2);
ndata = kj - 1; position = 0; optalphayg = 0; optnormy = 0; 
for ji=(ndata-1): -1:3
     if (norm_function(ji)<norm_function(ji-1)) & (norm_function(ji+1)>norm_function(ji))
     position = ji;
     optalphayg = arr_alpha(ji);
     optnormy = norm_function(ji);
     end;
end;
% plotting the L-norms for the y-derivative
if optalphayg ~= 0
   ynorm_3D=figure('Name','L norms for the y-derivative(3D)','NumberTitle','off');
   loglog(arr_alpha,norm_function1,arr_alpha,norm_function2,arr_alpha,norm_function,optalphayg,optnormy,...
           '--rs','LineWidth',1,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',10)
   xlabel('log10(alfa) []','FontSize',10); ylabel('log10Cnorm{reconstr(j)]-reconst(j-1)} [nT (or) mGal]','FontSize',10);
   legend('L_1 norm','L_2 norm','C = L_i_n_f norm','Location','South');
   title({'L norms for the y-derivative';'Press ''Z''key to enable picking mode or any key to disable picking mode'})
else
   ynorm_3D=figure('Name','L norms for the y-derivative(3D)','NumberTitle','off');
   loglog(arr_alpha,norm_function1,arr_alpha,norm_function2,arr_alpha,norm_function,optalphayg,optnormy,...
           '--rs','LineWidth',1,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',10)
   xlabel('log10(alfa) []','FontSize',10); ylabel('log10Cnorm{reconstr(j)]-reconst(j-1)} [nT (or) mGal]','FontSize',10);
   legend('L_1 norm','L_2 norm','C = L_i_n_f norm','Location','South');
   title({'L norms for the y-derivative, NO LOCAL MINIMUM FOUND';'Press ''Z''key to enable picking mode or any key to disable picking mode'});
end;

% set up the window position and calladditional functions
out_pos=get(ynorm_3D,'OuterPosition');scr=get(0,'ScreenSize');pos=get(ynorm_3D,'Position');
set(ynorm_3D,'Position',[scr(3)/2-out_pos(3) scr(4)/4 pos(3) pos(4)])

set(ynorm_3D,'keypressfcn',@normpick_2D)

y_deriv_3D(optalphayg,optalphayg);

% saving the C-norm-function
  for ji=1:ndata
       savearr(ji,1) = log10(arr_alpha(ji));
       savearr(ji,2) = log10(norm_function(ji));
  end;
  part1='Cnorm_yderiv_';part2=strrep(fnamegrid,'.grd','');part3= '.dat'; nametogether=strcat(part1,part2,part3);
  cd (pname_grid)
     savefile = fopen(nametogether, 'w');
     fprintf(savefile,'%s %s','alpha','Lnorm(inf)');
     fprintf(savefile, '\n%e %e', savearr');
     fclose(savefile);
     cd (c_dir_grid)

function z_derivat_norm_3D_Callback(src,eventdata)
global columnsn columnsnextr rowsm rowsmextr minx maxx miny maxy field extrapolfield optalphazg
global fnamegrid pname_grid c_dir_grid
%initial check
if isempty(field)
    helpdlg('Load grid data first','Wrong selection');
    return
else
end

%%%%%%%% EVALUATION OF THE NORM FOR THE REGULARIZED Z-DERIVATIVE (GRID)
% 2D direct FFT calculation
y = fft2(extrapolfield);

% input of regular. parameter limits in a window
 inform = 'evaluation of the norm (z-derivative)'; inform
 prompti = {'start value of regular. parameter (e.g. 1E0)','end value of regular. parameter (e.g. 1E+20)'};
 titlei = 'norm (z-derivative)';
 linesi = 1; defi = {'1E0','1E+20'};
 answer = inputdlg(prompti,titlei,linesi,defi);
 opergstr1 = answer(1); alphabeging3 = str2double(opergstr1);
 opergstr2 = answer(2); alphaendg3 = str2double(opergstr2);

% wave-numbers evaluation (saved in the fields fkx() and fky()  )
% dfx and dfy  are the steps of the wave-numbers
dx = abs(maxx-minx)/(columnsn-1); dy = abs(maxy-miny)/(rowsm-1);
dfx = 1/(dx*(columnsnextr-1)); dfy = 1/(dy*(rowsmextr-1));
R=rem(columnsnextr,2);
if R==0
       fkx(1:columnsnextr/2+1) = 0:dfx:(columnsnextr/2)*dfx;
       fkx_pom = -fkx(2:columnsnextr/2+1);
       fkx(columnsnextr/2+1:columnsnextr) = fkx_pom(:,[length(fkx_pom):-1:1]);
else
       fkx(1:round(columnsnextr/2))=0:dfx:floor(columnsnextr/2)*dfx;
       fkx_pom = -fkx(2:round(columnsnextr/2));
       fkx(round(columnsnextr/2)+1:columnsnextr) = fkx_pom(:,[length(fkx_pom):-1:1]);
end;
R=rem(rowsmextr,2);
if R==0
       fky(1:rowsmextr/2+1) = 0:dfy:(rowsmextr/2)*dfy;
       fky_pom = -fky(2:rowsmextr/2+1);
       fky(rowsmextr/2+1:rowsmextr) = fky_pom(:,[length(fky_pom):-1:1]);
else
       fky(1:round(rowsmextr/2))=0:dfy:floor(rowsmextr/2)*dfy;
       fky_pom = -fky(2:round(rowsmextr/2));
       fky(round(rowsmextr/2)+1:rowsmextr) = fky_pom(:,[length(fky_pom):-1:1]);
end;  
  
%MAIN CYCLE OF EVALUATION OF THE NORM FOR THE REGULARIZED Z-DERIVATIVE (GRID)
% initialisation of a help matrix - prevji() -it will contain previous reconstructed field (for previous alpha)
prevji = zeros(rowsmextr,columnsnextr);
% begin of the main cycle - for changing of alpha in a geometrical sequence
% setting the starting value of alpha,  kj is the order number for the increasing alpha
alpha = alphabeging3; kj = 1; alpha
% the final value of alpha is set as the limit of the main cycle
while alpha<alphaendg3
%alpha
   % evaluation of the z-horizontal derivative in spectral domain by means of the Tikhonov filter application
   for j=1:rowsmextr
       for k=1:columnsnextr  
        yt(j,k) = y(j,k)*sqrt(fkx(k)*fkx(k)+fky(j)*fky(j))*2*pi/(1+(alpha/(4*pi*pi))*(fkx(k)*fkx(k)+fky(j)*fky(j))*tanh(2*pi*sqrt(fkx(k)*fkx(k)+fky(j)*fky(j)))*tanh(2*pi*sqrt(fkx(k)*fkx(k)+fky(j)*fky(j))));
	   end;
   end;
   % inverse DFT
   B = real(ifft2(yt));

% Cnorm-function calculation
   maxdiffer = 0;
   for j=2:rowsmextr-1
     for k=2:columnsnextr-1
       if abs(B(j,k)-prevji(j,k))>maxdiffer
          maxdiffer = abs(B(j,k)-prevji(j,k)); 
       end;
     end;
   end;
   arr_alpha(kj) = alpha;
   norm_function(kj) = maxdiffer;
   %L(p)-norms evaluation
   norm_function1(kj) = norm(B-prevji,1); norm_function2(kj) = norm(B-prevji,2);
   norm_functionC(kj) = norm(B-prevji,inf); norm_function_Fro(kj) = norm(B-prevji,'fro');
   % filling of the help matrix prevji() with the last actual solution (for last actual alpha)
   prevji = B;
% increasing of alpha (in a geometrical sequence, with a quocient of 1.1)
alpha = 1.1*alpha; kj = kj + 1;
end; 
kj = kj - 2; alpha
% end of the main cycle - for changing of alpha in a geometrical sequence

% finding of local minimum of the C norm function for the z-derivative
% the first value of the norm function can not be evaluated correctly, it is taken equal to the second value
norm_function(1) = norm_function(2);
ndata = kj - 1; position = 0; optalphazg = 0; optnormz = 0;
for ji=(ndata-1): -1:3
     if (norm_function(ji)<norm_function(ji-1)) & (norm_function(ji+1)>norm_function(ji))
     position = ji;
     optalphazg = arr_alpha(ji);
     optnormz = norm_function(ji);
     end;
end;
% plotting the L-norms for the z-derivative
if optalphazg ~= 0
   znorm_3D=figure('Name','L norms for the z-derivative(3D)','NumberTitle','off');
   loglog(arr_alpha,norm_function1,arr_alpha,norm_function2,arr_alpha,norm_function,optalphazg,optnormz,...
           '--rs','LineWidth',1,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',10)
   xlabel('log10(alfa) []','FontSize',10); ylabel('log10Cnorm{reconstr(j)]-reconst(j-1)} [nT (or) mGal]','FontSize',10);
   legend('L_1 norm','L_2 norm','C = L_i_n_f norm','Location','South');
   title({'L norms for the z-derivative';'Press ''Z''key to enable picking mode or any key to disable picking mode'});
else
   znorm_3D=figure('Name','L norms for the z-derivative(3D)','NumberTitle','off');
   loglog(arr_alpha,norm_function1,arr_alpha,norm_function2,arr_alpha,norm_function,optalphazg,optnormz,...
           '--rs','LineWidth',1,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',10)
   xlabel('log10(alfa) []','FontSize',10); ylabel('log10Cnorm{reconstr(j)]-reconst(j-1)} [nT (or) mGal]','FontSize',10);
   legend('L_1 norm','L_2 norm','C = L_i_n_f norm','Location','South');
   title({'L norms for the z-derivative, NO LOCAL MINIMUM FOUND';'Press ''Z''key to enable picking mode or any key to disable picking mode'});
end;

% set up the window position and calladditional functions
out_pos=get(znorm_3D,'OuterPosition');scr=get(0,'ScreenSize');pos=get(znorm_3D,'Position');
set(znorm_3D,'Position',[scr(3)/2-out_pos(3) (scr(4)/4)-20 pos(3) pos(4)])

set(znorm_3D,'keypressfcn',@normpick_2D)

z_deriv_3D(optalphazg,optalphazg);

% saving the C-norm-function
  for ji=1:ndata
       savearr(ji,1) = log10(arr_alpha(ji));
       savearr(ji,2) = log10(norm_function(ji));
  end;
  part1='Cnorm_zderiv_';part2=strrep(fnamegrid,'.grd','');part3= '.dat'; nametogether=strcat(part1,part2,part3);
  cd (pname_grid)
     savefile = fopen(nametogether, 'w');
     fprintf(savefile,'%s %s','alpha','Lnorm(inf)');
     fprintf(savefile, '\n%e %e', savearr');
     fclose(savefile);
     cd (c_dir_grid)

% --------------------------------------------------------------------
function x_derivat_calc_3D_Callback(src,eventdata)
global optalphaxg field
%initial check
if isempty(field)
    helpdlg('Load grid data first','Wrong selection');
    return
else
end

%%%%%%%% FINAL X-DERIVATIVE (GRID) EVALUATION
% input of regular. parameter limits in a window
 prompti = {'optimum value of regular. parameter (0 for non-regular.)'};
 titlei = 'set parameter';
 linesi = 1; defi = {num2str(optalphaxg)};
 answer = inputdlg(prompti,titlei,linesi,defi);
 opergstr1 = answer(1); 
 optalphaxg = str2double(opergstr1);
 x_deriv_3D(optalphaxg);
 
function x_deriv_3D(varargin)
global columnsn columnsnextr rowsm rowsmextr numextrRows numextrCols minx maxx miny maxy extrapolfield fnamegrid xax yax
global fig_xder_3D xder_3D fig_xder_3D_cont optalphaxg grid_regdx c_dir_grid pname_grid

if length(varargin)==2
   optalphaxg=varargin{1};pick_alpha=varargin{2};optalphaxg=pick_alpha;
elseif length(varargin)==1
    optalphaxg=varargin{1};
end

% 2D direct FFT calculation
y = fft2(extrapolfield);

% wave-numbers evaluation (saved in the fields fkx() and fky()  )
% dfx and dfy  are the steps of the wave-numbers
dx = abs(maxx-minx)/(columnsn-1); dy = abs(maxy-miny)/(rowsm-1);
dfx = 1/(dx*(columnsnextr-1)); dfy = 1/(dy*(rowsmextr-1));
R=rem(columnsnextr,2);
if R==0
       fkx(1:columnsnextr/2+1) = 0:dfx:(columnsnextr/2)*dfx;
       fkx_pom = -fkx(2:columnsnextr/2+1);
       fkx(columnsnextr/2+1:columnsnextr) = fkx_pom(:,[length(fkx_pom):-1:1]);
else
       fkx(1:round(columnsnextr/2))=0:dfx:floor(columnsnextr/2)*dfx;
       fkx_pom = -fkx(2:round(columnsnextr/2));
       fkx(round(columnsnextr/2)+1:columnsnextr) = fkx_pom(:,[length(fkx_pom):-1:1]);
end;
R=rem(rowsmextr,2);
if R==0
       fky(1:rowsmextr/2+1) = 0:dfy:(rowsmextr/2)*dfy;
       fky_pom = -fky(2:rowsmextr/2+1);
       fky(rowsmextr/2+1:rowsmextr) = fky_pom(:,[length(fky_pom):-1:1]);
else
       fky(1:round(rowsmextr/2))=0:dfy:floor(rowsmextr/2)*dfy;
       fky_pom = -fky(2:round(rowsmextr/2));
       fky(round(rowsmextr/2)+1:rowsmextr) = fky_pom(:,[length(fky_pom):-1:1]);
end;  
  
% calculation of the final x-derived field (for the optimum or selected value of alpha)
   for j=1:rowsmextr
       for k=1:columnsnextr
        yt(j,k) = y(j,k)*sqrt(-1)*fkx(k)*2*pi/(1+(optalphaxg/(4*pi*pi))*(fkx(k)*fkx(k)+fky(j)*fky(j))*tanh(2*pi*sqrt(fkx(k)*fkx(k)+fky(j)*fky(j)))*tanh(2*pi*sqrt(fkx(k)*fkx(k)+fky(j)*fky(j))));
	   end;
   % inverse DFT
   end;
   finaldx = real(ifft2(yt));
   
%exctracting back to the original size of the input grid
finaldx2 = finaldx(numextrRows+1:rowsmextr-numextrRows,numextrCols+1:columnsnextr-numextrCols);

% saving the final x-derived data (when the optimum alpha is not equal to the last used alpha)
% EXPORT OF THE GRID FILE IN THE SURFER ASCII *.GRD FORMAT
% evaluation of the important patameters for the GS ASCII grid export
minv = min(min(finaldx2)); maxv=max(max(finaldx2));

 cd (pname_grid)  
 if optalphaxg ~= 0
    %creating of the outputfile name - in the first part using the name of the input grid
    %one of trials = part2=strrep(fnamegrid,'.grd','');
    part1='x-regderiv_';part2=fnamegrid; nametogether=strcat(part1,part2);
    fid = fopen(nametogether,'w');
 else
    %creating of the outputfile name - in the first part using the name of the input grid
    part1='x-nonregder_';part2=fnamegrid; nametogether=strcat(part1,part2);
    fid = fopen(nametogether,'w');
 end;
fprintf(fid,'%s\n','DSAA');
fprintf(fid,'%3.0f %3.0f\n',columnsn,rowsm);
fprintf(fid,'%f %f\n',minx,maxx);fprintf(fid,'%f %f\n',miny,maxy);fprintf(fid,'%e %e\n',minv,maxv);
fprintf(fid,'%e ',finaldx2');
status = fclose(fid);
cd (c_dir_grid)
% plotting the contour map of the final x-derived data (when the optimum alpha is not equal to the last used alpha)
if length(varargin)==2
   
   if pick_alpha>99999
      title_var = '%.3e';
   else
      title_var = '%.3f';
   end
   
   if ishandle(findobj('Name','regularized x-derivative(3D)'))
        title_solid=('output - regularized x-derivative, picked alpha value:');
        title_format=strcat(title_solid,title_var);title_str=sprintf(title_format, pick_alpha);
        set(fig_xder_3D,'Cdata',finaldx2);set(fig_xder_3D_cont,'Zdata',finaldx2)
        if pick_alpha==0
           title_str=('output - NON-regular. x-derivative (alpha = 0)');
        end
        title(get(fig_xder_3D,'Parent'),title_str)
        colormap(gca(xder_3D),colscale)
   else
        title_solid=('output - regularized x-derivative, picked alpha value:');
        title_format=strcat(title_solid,title_var);title_str=sprintf(title_format, pick_alpha);
        xder_3D=figure('Name','regularized x-derivative(3D)','NumberTitle','off');
        fig_xder_3D=pcolor(xax,yax,finaldx2);colorbar;legend('x-derived');shading interp 
        xlabel('x [dist. units]','FontSize',10); ylabel('y [dist. units]','FontSize',10);
        if pick_alpha==0
           title_str=('output - NON-regular. x-derivative (alpha = 0)');
        end
        title(title_str);hold on;[C,fig_xder_3D_cont]=contour(xax,yax,finaldx2,'k');hold off;
        colormap(gca(xder_3D),colscale)
        set(xder_3D,'WindowButtonMotionFcn',@cursorcoordinate)
   end   

else

    if optalphaxg ~= 0
       xder_3D=figure('Name','regularized x-derivative(3D)','NumberTitle','off');
       fig_xder_3D=pcolor(xax,yax,finaldx2);colorbar;legend('x-derived');shading interp 
       xlabel('x [dist. units]','FontSize',10); ylabel('y [dist. units]','FontSize',10);
       title('output - regularized x-derivative (select y-deriv. calc.)       [unit/m]');
       hold on;[C,fig_xder_3D_cont]=contour(xax,yax,finaldx2,'k');hold off;
       colormap(gca(xder_3D),colscale)
       set(xder_3D,'WindowButtonMotionFcn',@cursorcoordinate)
    else
       xder_3D=figure('Name','NON-regularized x-derivative(3D)','NumberTitle','off');
       fig_xder_3D=pcolor(xax,yax,finaldx2);colorbar; legend('x-derived'); shading interp
       xlabel('x [dist. units]','FontSize',10); ylabel('y [dist. units]','FontSize',10);
       title('output - NON-regularized x-derivative (used alpha = 0) (select y-deriv. calc.)  [unit/m]');
       hold on;[C,fig_xder_3D_cont]=contour(xax,yax,finaldx2,'k');hold off;
       colormap(gca(xder_3D),colscale)
       set(xder_3D,'WindowButtonMotionFcn',@cursorcoordinate)
    end;
end

out_pos=get(xder_3D,'OuterPosition');scr=get(0,'ScreenSize');pos=get(xder_3D,'Position');
set(xder_3D,'Position',[scr(3)/2 (scr(4)/4)+20 pos(3) pos(4)])

%for the use in 3D Euler deconvolution algorithm
grid_regdx = finaldx2;

% --------------------------------------------------------------------
function y_derivat_calc_3D_Callback(src,eventdata)
global optalphayg field

%initial check
if isempty(field)
    helpdlg('Load grid data first','Wrong selection');
    return
else
end
%%%%%%%% FINAL Y-DERIVATIVE (GRID) EVALUATION
% input of regular. parameter limits in a window
 prompti = {'optimum value of regular. parameter (0 for non-regular.)'};
 titlei = 'set parameter';
 linesi = 1; defi = {num2str(optalphayg)};
 answer = inputdlg(prompti,titlei,linesi,defi);
 opergstr1 = answer(1); 
 optalphayg = str2double(opergstr1);
 y_deriv_3D(optalphayg);
 
function y_deriv_3D(varargin)
global columnsn columnsnextr rowsm rowsmextr numextrRows numextrCols minx maxx miny maxy field extrapolfield fnamegrid xax yax
global fig_yder_3D yder_3D fig_yder_3D_cont optalphayg grid_regdy c_dir_grid pname_grid
  
if length(varargin)==2
    optalphayg=varargin{1};pick_alpha=varargin{2};optalphayg=pick_alpha;
elseif length(varargin)==1
    optalphayg=varargin{1};
end    
% 2D direct FFT calculation
y = fft2(extrapolfield);

% wave-numbers evaluation (saved in the fields fkx() and fky()  )
% dfx and dfy  are the steps of the wave-numbers
dx = abs(maxx-minx)/(columnsn-1); dy = abs(maxy-miny)/(rowsm-1);
dfx = 1/(dx*(columnsnextr-1)); dfy = 1/(dy*(rowsmextr-1));
R=rem(columnsnextr,2);
if R==0
       fkx(1:columnsnextr/2+1) = 0:dfx:(columnsnextr/2)*dfx;
       fkx_pom = -fkx(2:columnsnextr/2+1);
       fkx(columnsnextr/2+1:columnsnextr) = fkx_pom(:,[length(fkx_pom):-1:1]);
else
       fkx(1:round(columnsnextr/2))=0:dfx:floor(columnsnextr/2)*dfx;
       fkx_pom = -fkx(2:round(columnsnextr/2));
       fkx(round(columnsnextr/2)+1:columnsnextr) = fkx_pom(:,[length(fkx_pom):-1:1]);
end;
R=rem(rowsmextr,2);
if R==0
       fky(1:rowsmextr/2+1) = 0:dfy:(rowsmextr/2)*dfy;
       fky_pom = -fky(2:rowsmextr/2+1);
       fky(rowsmextr/2+1:rowsmextr) = fky_pom(:,[length(fky_pom):-1:1]);
else
       fky(1:round(rowsmextr/2))=0:dfy:floor(rowsmextr/2)*dfy;
       fky_pom = -fky(2:round(rowsmextr/2));
       fky(round(rowsmextr/2)+1:rowsmextr) = fky_pom(:,[length(fky_pom):-1:1]);
end;  
  
% calculation of the final y-derived field (for the optimum or selected value of alpha)
   for j=1:rowsmextr
       for k=1:columnsnextr
	    yt(j,k) = y(j,k)*sqrt(-1)*fky(j)*2*pi/(1+(optalphayg/(4*pi*pi))*(fkx(k)*fkx(k)+fky(j)*fky(j))*tanh(2*pi*sqrt(fkx(k)*fkx(k)+fky(j)*fky(j)))*tanh(2*pi*sqrt(fkx(k)*fkx(k)+fky(j)*fky(j))));
       end;
   end;
   % inverse DFT
   finaldy = real(ifft2(yt));

%exctracting back to the original size of the input grid
finaldy2 = finaldy(numextrRows+1:rowsmextr-numextrRows,numextrCols+1:columnsnextr-numextrCols);

% saving the final y-derived data (when the optimum alpha is not equal to the last used alpha)
% EXPORT OF THE GRID FILE IN THE SURFER ASCII *.GRD FORMAT
minv = min(min(finaldy2)); maxv=max(max(finaldy2));

 cd (pname_grid)  
 if optalphayg ~= 0
    %creating of the outputfile name - in the first part using the name of the input grid
    part1='y-regderiv_';part2=fnamegrid; nametogether=strcat(part1,part2);
    fid = fopen(nametogether,'w');
 else
    %creating of the outputfile name - in the first part using the name of the input grid
    part1='y-nonregder_';part2=fnamegrid; nametogether=strcat(part1,part2);
    fid = fopen(nametogether,'w');
 end;
fprintf(fid,'%s\n','DSAA');
fprintf(fid,'%3.0f %3.0f\n',columnsn,rowsm);
fprintf(fid,'%f %f\n',minx,maxx);fprintf(fid,'%f %f\n',miny,maxy);fprintf(fid,'%e %e\n',minv,maxv);
fprintf(fid,'%e ',finaldy2');
status = fclose(fid);
cd(c_dir_grid)

% plotting the contour map of the final y-derived data (when the optimum alpha is not equal to the last used alpha)

if length(varargin)==2
   
   if pick_alpha>99999
      title_var = '%.3e';
   else
      title_var = '%.3f';
   end
   
   if ishandle(findobj('Name','regularized y-derivative(3D)'))
        title_solid=('output - regularized y-derivative, picked alpha value:');
        title_format=strcat(title_solid,title_var);title_str=sprintf(title_format, pick_alpha);
        set(fig_yder_3D,'Cdata',finaldy2);set(fig_yder_3D_cont,'Zdata',finaldy2)
        if pick_alpha==0
           title_str=('output - NON-regular. y-derivative (alpha = 0)');
        end
        title(get(fig_yder_3D,'Parent'),title_str)
        colormap(gca(yder_3D),colscale)
   else
        title_solid=('output - regularized y-derivative, picked alpha value:');
        title_format=strcat(title_solid,title_var);title_str=sprintf(title_format, pick_alpha);
        yder_3D=figure('Name','regularized y-derivative(3D)','NumberTitle','off');
        fig_yder_3D=pcolor(xax,yax,finaldy2);colorbar;legend('y-derived');shading interp 
        xlabel('x [dist. units]','FontSize',10); ylabel('y [dist. units]','FontSize',10);
        if pick_alpha==0
           title_str=('output - NON-regular. y-derivative (alpha = 0)');
        end
        title(title_str);hold on;[C,fig_yder_3D_cont]=contour(xax,yax,finaldy2,'k');hold off;
        colormap(gca(yder_3D),colscale)
        set(yder_3D,'WindowButtonMotionFcn',@cursorcoordinate)
   end   

else

    if optalphayg ~= 0
       yder_3D=figure('Name','regularized y-derivative(3D)','NumberTitle','off');
       fig_yder_3D=pcolor(xax,yax,finaldy2);colorbar;legend('y-derived');shading interp 
       xlabel('x [dist. units]','FontSize',10); ylabel('y [dist. units]','FontSize',10);
       title('output - regularized y-derivative (select y-deriv. calc.)       [unit/m]');
       hold on;[C,fig_yder_3D_cont]=contour(xax,yax,finaldy2,'k');hold off;
       colormap(gca(yder_3D),colscale)
       set(yder_3D,'WindowButtonMotionFcn',@cursorcoordinate)
    else
       yder_3D=figure('Name','NON-regularized y-derivative(3D)','NumberTitle','off');
       fig_yder_3D=pcolor(xax,yax,finaldy2);colorbar; legend('y-derived'); shading interp
       xlabel('x [dist. units]','FontSize',10); ylabel('y [dist. units]','FontSize',10);
       title('output - NON-regularized y-derivative (used alpha = 0) (select y-deriv. calc.)  [unit/m]');
       hold on;[C,fig_yder_3D_cont]=contour(xax,yax,finaldy2,'k');hold off;
       colormap(gca(yder_3D),colscale)
       set(yder_3D,'WindowButtonMotionFcn',@cursorcoordinate)
    end;
end

out_pos=get(yder_3D,'OuterPosition');scr=get(0,'ScreenSize');pos=get(yder_3D,'Position');
set(yder_3D,'Position',[scr(3)/2 scr(4)/4 pos(3) pos(4)])


%for the use in 3D Euler deconvolution algorithm
grid_regdy = finaldy2;

% --------------------------------------------------------------------
function z_derivat_calc_3D_Callback(src,eventdata)
global optalphazg field

%initial check
if isempty(field)
    helpdlg('Load grid data first','Wrong selection');
    return
else
end
%%%%%%%% FINAL Z-DERIVATIVE (GRID) EVALUATION
% input of regular. parameter limits in a window
 prompti = {'optimum value of regular. parameter (0 for non-regular.)'};
 titlei = 'set parameter';
 linesi = 1; defi = {num2str(optalphazg)};
 answer = inputdlg(prompti,titlei,linesi,defi);
 opergstr1 = answer(1); optalphazg = str2double(opergstr1);
 z_deriv_3D(optalphazg);

function z_deriv_3D(varargin)
global columnsn columnsnextr rowsm rowsmextr numextrRows numextrCols minx maxx miny maxy field extrapolfield fnamegrid xax yax
global fig_zder_3D zder_3D fig_zder_3D_cont optalphazg grid_regdz c_dir_grid pname_grid
 
if length(varargin)==2
    optalphazg=varargin{1};pick_alpha=varargin{2};optalphazg=pick_alpha;
elseif length(varargin)==1
    optalphazg=varargin{1};
end    
% 2D direct FFT calculation
y = fft2(extrapolfield);

% wave-numbers evaluation (saved in the fields fkx() and fky()  )
% dfx and dfy  are the steps of the wave-numbers
dx = abs(maxx-minx)/(columnsn-1); dy = abs(maxy-miny)/(rowsm-1);
dfx = 1/(dx*(columnsnextr-1)); dfy = 1/(dy*(rowsmextr-1));
R=rem(columnsnextr,2);
if R==0
       fkx(1:columnsnextr/2+1) = 0:dfx:(columnsnextr/2)*dfx;
       fkx_pom = -fkx(2:columnsnextr/2+1);
       fkx(columnsnextr/2+1:columnsnextr) = fkx_pom(:,[length(fkx_pom):-1:1]);
else
       fkx(1:round(columnsnextr/2))=0:dfx:floor(columnsnextr/2)*dfx;
       fkx_pom = -fkx(2:round(columnsnextr/2));
       fkx(round(columnsnextr/2)+1:columnsnextr) = fkx_pom(:,[length(fkx_pom):-1:1]);
end;
R=rem(rowsmextr,2);
if R==0
       fky(1:rowsmextr/2+1) = 0:dfy:(rowsmextr/2)*dfy;
       fky_pom = -fky(2:rowsmextr/2+1);
       fky(rowsmextr/2+1:rowsmextr) = fky_pom(:,[length(fky_pom):-1:1]);
else
       fky(1:round(rowsmextr/2))=0:dfy:floor(rowsmextr/2)*dfy;
       fky_pom = -fky(2:round(rowsmextr/2));
       fky(round(rowsmextr/2)+1:rowsmextr) = fky_pom(:,[length(fky_pom):-1:1]);
end;  

% calculation of the final z-derived field (for the optimum or selected value of alpha)
   for j=1:rowsmextr
       for k=1:columnsnextr
        yt(j,k) = y(j,k)*sqrt(fkx(k)*fkx(k)+fky(j)*fky(j))*2*pi/(1+(optalphazg/(4*pi*pi))*(fkx(k)*fkx(k)+fky(j)*fky(j))*tanh(2*pi*sqrt(fkx(k)*fkx(k)+fky(j)*fky(j)))*tanh(2*pi*sqrt(fkx(k)*fkx(k)+fky(j)*fky(j))));
	   end;
   end;
   % inverse DFT
   finaldz = real(ifft2(yt));

%exctracting back to the original size of the input grid
finaldz2 = finaldz(numextrRows+1:rowsmextr-numextrRows,numextrCols+1:columnsnextr-numextrCols);

% saving the final z-derived data (when the optimum alpha is not equal to the last used alpha)
% EXPORT OF THE GRID FILE IN THE SURFER ASCII *.GRD FORMAT

minv = min(min(finaldz2)); maxv=max(max(finaldz2));

cd(pname_grid)   
if optalphazg ~= 0
    %creating of the outputfile name - in the first part using the name of the input grid
    part1='z-regderiv_';part2=fnamegrid; nametogether=strcat(part1,part2);
    fid = fopen(nametogether,'w');
 else
    %creating of the outputfile name - in the first part using the name of the input grid
    part1='z-nonregder_';part2=fnamegrid; nametogether=strcat(part1,part2);
    fid = fopen(nametogether,'w');
 end;
fprintf(fid,'%s\n','DSAA');
fprintf(fid,'%3.0f %3.0f\n',columnsn,rowsm);
fprintf(fid,'%f %f\n',minx,maxx);fprintf(fid,'%f %f\n',miny,maxy);fprintf(fid,'%e %e\n',minv,maxv);
fprintf(fid,'%e ',finaldz2');
status = fclose(fid);
cd(c_dir_grid)
% plotting the contour map of the final z-derived data (when the optimum alpha is not equal to the last used alpha)
if length(varargin)==2
   
   if pick_alpha>99999
      title_var = '%.3e';
   else
      title_var = '%.3f';
   end
   
   if ishandle(findobj('Name','regularized z-derivative(3D)'))
        title_solid=('output - regularized z-derivative, picked alpha value:');
        title_format=strcat(title_solid,title_var);title_str=sprintf(title_format, pick_alpha);
        set(fig_zder_3D,'Cdata',finaldz2);set(fig_zder_3D_cont,'Zdata',finaldz2)
        if pick_alpha==0
           title_str=('output - NON-regular. z-derivative (alpha = 0)');
        end
        title(get(fig_zder_3D,'Parent'),title_str)
        colormap(gca(zder_3D),colscale)
   else
        title_solid=('output - regularized z-derivative, picked alpha value:');
        title_format=strcat(title_solid,title_var);title_str=sprintf(title_format, pick_alpha);
        zder_3D=figure('Name','regularized z-derivative(3D)','NumberTitle','off');
        fig_zder_3D=pcolor(xax,yax,finaldz2);colorbar;legend('z-derived');shading interp 
        xlabel('x [dist. units]','FontSize',10); ylabel('y [dist. units]','FontSize',10);
        if pick_alpha==0
           title_str=('output - NON-regular. z-derivative (alpha = 0)');
        end
        title(title_str);hold on;[C,fig_zder_3D_cont]=contour(xax,yax,finaldz2,'k');hold off;
        colormap(gca(zder_3D),colscale)
        set(zder_3D,'WindowButtonMotionFcn',@cursorcoordinate)
   end   

else

    if optalphazg ~= 0
       zder_3D=figure('Name','regularized z-derivative(3D)','NumberTitle','off');
       fig_zder_3D=pcolor(xax,yax,finaldz2);colorbar;legend('z-derived');shading interp 
       xlabel('x [dist. units]','FontSize',10); ylabel('y [dist. units]','FontSize',10);
       title('output - regularized z-derivative (select y-deriv. calc.)       [unit/m]');
       hold on;[C,fig_zder_3D_cont]=contour(xax,yax,finaldz2,'k');hold off;
       colormap(gca(zder_3D),colscale)
       set(zder_3D,'WindowButtonMotionFcn',@cursorcoordinate)
    else
       zder_3D=figure('Name','NON-regularized z-derivative(3D)','NumberTitle','off');
       fig_zder_3D=pcolor(xax,yax,finaldz2);colorbar; legend('z-derived'); shading interp
       xlabel('x [dist. units]','FontSize',10); ylabel('y [dist. units]','FontSize',10);
       title('output - NON-regularized z-derivative (used alpha = 0) (select y-deriv. calc.)  [unit/m]');
       hold on;[C,fig_zder_3D_cont]=contour(xax,yax,finaldz2,'k');hold off;
       colormap(gca(zder_3D),colscale)
       set(zder_3D,'WindowButtonMotionFcn',@cursorcoordinate)
    end;
end

out_pos=get(zder_3D,'OuterPosition');scr=get(0,'ScreenSize');pos=get(zder_3D,'Position');
set(zder_3D,'Position',[scr(3)/2 (scr(4)/4)-20 pos(3) pos(4)])
%for the use in 3D Euler deconvolution algorithm
grid_regdz = finaldz2;

%3. PROFILE EULER DECONVOLUTION~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

function Euler_deconvolution_2D_Callback(src,eventdata)
global profEDwind profED_Nstr profED_polyn
global F prof_regdx prof_regdz X m 
global psol_i px_sol0 pz_sol0 pz_sol0_SD pc_sol0 per_sol0 c_dir_prof pname_prof fnameprofile
global fig_SD fig_CN fig_CE info_line_prof

%Initial check
%data check
if isempty(F)
    helpdlg('Load profile data first','Wrong selection');
    return
end

%derivatives check
if isempty(prof_regdx)||isempty(prof_regdz)
    helpdlg('Compute x & z derivatives first','Wrong selection');
    return
end

% INPUT OF THE PARAMETERS OF THE PROFILE EULER DECONVOLUTION

 prompti = {'profile window size for ED (e.g. 10)','structural index: 0(contact-mag,dike-grav), 1(dike-mag,cylin.-grav),       2(cylin.-mag)','interference polynomial degree (0, 1 or 2) [0 for classical approach]'};
 titlei = 'parameters of profile (2D) Euler deconvolution';
 linesi = 1; defi = {num2str(profEDwind),num2str(profED_Nstr),num2str(profED_polyn)};
 answer = inputdlg(prompti,titlei,linesi,defi);
 opergstr1 = answer(1); profEDwind = str2double(opergstr1);
 opergstr2 = answer(2); profED_Nstr = str2double(opergstr2);
 opergstr3 = answer(3); profED_polyn = str2double(opergstr3);


%clear old Euler deconv. solutions
psol_i=[]; px_sol0=[]; pz_sol0=[]; pz_sol0_SD=[]; pc_sol0=[]; per_sol0=[];

%clear old selection criteria plots and data
if ishandle(fig_SD)
    close(fig_SD)
    clear get(findobj('Tag','SD_p'),'UserData')
end
if ishandle(fig_CN)
   close(fig_CN)
   clear get(findobj('Tag','CN_p'),'UserData')
end
if ishandle(fig_CE)
    close(fig_CE)
    clear get(findobj('Tag','CE_p'),'UserData')
end

% 2D EULER DECONV. ALGORITHM (Thompson, 1982-equation format [right-hand side: -N(f-B)])

operl = profEDwind; NSI = profED_Nstr; Xp = X; finaldx = prof_regdx; finaldz = prof_regdz;
% number of unknown parameters = 3 (x0,z0,B)
% main cycle along the profile data, sol_i - order number of obtained solutions
psol_i=0;
for i=1:m-operl-1
        % creation of the matrix for 4 stages of the interpr. equation
        for k=0:operl-1
            switch profED_polyn
                case 0
                A(k+1,1)=finaldx(i+k); A(k+1,2)=finaldz(i+k); A(k+1,3)=NSI;
                case 1
                A(k+1,1)=finaldx(i+k); A(k+1,2)=finaldz(i+k); A(k+1,3)=1; A(k+1,4)=Xp(i+k);
                case 2
                A(k+1,1)=finaldx(i+k); A(k+1,2)=finaldz(i+k); A(k+1,3)=1; A(k+1,4)=Xp(i+k); A(k+1,5)=Xp(i+k)*Xp(i+k);
            end;
        end; 
         % creation of the right-hand side vector b
         for k=0:operl-1
             b(k+1,1) = Xp(i+k)*finaldx(i+k)+NSI*F(i+k);
         end;
        % solution of the Ax = b system - by means of Moore-Penrose method
        xs = pinv(A)*b;
        % evaluation of the condition number of the solution
        c = cond(A);
        % inverse of the normal equation matrix (for the evaluation of the stand. deviation of the solution)
        iA = inv(A'*A);
        % number of solutions will be increased
        psol_i = psol_i + 1;
        % calculation of parameters x0 and z0
        px_sol0(psol_i,1) = xs(1);  px_sol0_SD(psol_i,1) = sqrt(iA(1,1));
        pz_sol0(psol_i,1) = -xs(2); pz_sol0_SD(psol_i,1) = sqrt(iA(2,2));
        switch profED_polyn
                case 0
                pB_sol0(psol_i,1) = xs(3)/(NSI+1E-6); pA0_sol0(psol_i,1) = xs(3);
                case 1
                pB_sol0(psol_i,1) = xs(3)/(NSI+1E-6); pA0_sol0(psol_i,1) = xs(3); pA1_sol0(psol_i,1) = xs(4)/(NSI+1E-6);
                case 2
                pB_sol0(psol_i,1) = xs(3)/(NSI+1E-6); pA0_sol0(psol_i,1) = xs(3); pA1_sol0(psol_i,1) = xs(4)/(NSI+1E-6); pA2_sol0(psol_i,1) = xs(5)/(NSI+1E-6);
        end;
        % evaluation of the condition number of the solution
	    pc_sol0(psol_i,1) = c; 
		% evaluation of the ED-equation error (based on Cooper,2004 idea)
		switch profED_polyn
                case 0
                psumer = 0;
                for k=0:operl-1
                     psumer = psumer + xs(1)*finaldx(i+k)+xs(2)*finaldz(i+k)+xs(3)*NSI-Xp(i+k)*finaldx(i+k)-NSI*F(i+k);
                end; per_sol0(psol_i,1) = psumer;
                case 1
                psumer = 0;
                for k=0:operl-1
                     psumer = psumer + xs(1)*finaldx(i+k)+xs(2)*finaldz(i+k)+xs(3)*NSI+xs(4)*Xp(i+k)-Xp(i+k)*finaldx(i+k)-NSI*F(i+k);
                end; per_sol0(psol_i,1) = psumer;
                case 2
                psumer = 0;
                for k=0:operl-1
                     psumer = psumer + xs(1)*finaldx(i+k)+xs(2)*finaldz(i+k)+xs(3)*NSI+xs(4)*Xp(i+k)+xs(5)*Xp(i+k)*Xp(i+k)-Xp(i+k)*finaldx(i+k)-NSI*F(i+k);
                end; per_sol0(psol_i,1) = psumer;
        end; 
        % if the solution for x0 is not in the interval of the profile, or when z0 is a positive number or if the solution for z0 is not pure real number, it will be not accepted
        if ( px_sol0(psol_i,1)<Xp(i) ) | ( px_sol0(psol_i,1)>Xp(i+operl-1) ) | ( imag(pz_sol0(psol_i,1))~=0 ) | (pz_sol0(psol_i,1)>0)
            psol_i=psol_i - 1;
        end;
end; % end of the main cycle along the profile data,
all_2DEuler_solutions = psol_i;
all_2DEuler_solutions

% plotting of obtained standard deviations 
fig_SD=figure('Name','standard deviation of the depth solution','NumberTitle','off');
plot(px_sol0,pz_sol0_SD,'--rs','LineWidth',2,'MarkerEdgeColor','k','MarkerFaceColor','r','MarkerSize',6);
axis([Xp(1) Xp(m) min(pz_sol0_SD) max(pz_sol0_SD)]); 
xlabel('x [dist. units]','FontSize',10); ylabel('standard deviation of the depth solution [dist. units]','FontSize',10);title('selection of 2D-Euler solutions (standard deviation)');
set(fig_SD,'WindowButtonMotionFcn',@cursorcoordinate)
selectionLine(fig_SD);
set(gca(fig_SD),'Tag','SD_p');

% plotting of obtained condition numbers 
fig_CN=figure('Name','condition number of the depth solution','NumberTitle','off');
plot(px_sol0,pc_sol0,'--rs','LineWidth',2,'MarkerEdgeColor','k','MarkerFaceColor','b','MarkerSize',6);
axis([Xp(1) Xp(m) min(pc_sol0) max(pc_sol0)]); 
xlabel('x [dist. units]','FontSize',10); ylabel('condition number of the depth solution []','FontSize',10);title('selection of 2D-Euler solutions (condition number)');
set(fig_CN,'WindowButtonMotionFcn',@cursorcoordinate)
selectionLine(fig_CN);
set(gca(fig_CN),'Tag','CN_p');

% plotting of obtained "Cooper's errors" 
fig_CE=figure('Name','errors in Cooper''s sense','NumberTitle','off');
plot(px_sol0,abs(per_sol0),'--rs','LineWidth',2,'MarkerEdgeColor','k','MarkerFaceColor','m','MarkerSize',6);
axis([Xp(1) Xp(m) min(abs(per_sol0)) max(abs(per_sol0))]); 
xlabel('x [dist. units]','FontSize',10); ylabel('fit errors (in Coopers sense) of the depth solutions [nT (or) mGal] ','FontSize',10);title('selection of 2D-Euler solutions (solution fit)');
set(fig_CE,'WindowButtonMotionFcn',@cursorcoordinate)
selectionLine(fig_CE);
set(gca(fig_CE),'Tag','CE_p');

minzsol0 = min(pz_sol0);
% plotting of all obtained depth solutions 
% plot(px_sol0,pz_sol0)
fig_sol=figure('Name','2D-Euler depth solutions','NumberTitle','off');
plot(px_sol0,pz_sol0,'--rs','LineWidth',2,'MarkerEdgeColor','k','MarkerFaceColor','k','MarkerSize',6)
axis([Xp(1) Xp(m) minzsol0 0]); 
xlabel('x [dist. units]','FontSize',10); ylabel('depth [dist. units]','FontSize',10);
info_line_prof = sprintf('ED parameters: Window size: %.0f,Structural index: %.0f, Polynomial: %.0f',profEDwind,profED_Nstr,profED_polyn);
title({'all 2D-Euler depth solutions (saved in Euler-2D-solutions-all.dat file)',info_line_prof});

set(fig_sol,'WindowButtonMotionFcn',@cursorcoordinate)
% writing of determined solutions into a file
% creating a matrix from solution vectors
solXZ = [px_sol0'; pz_sol0'; pz_sol0_SD'; pc_sol0'; abs(per_sol0)'];

cd(pname_prof)
part1='euler_2D_sol_all_';part2=fnameprofile;nametogether=strcat(part1,part2);
savefile = fopen(nametogether, 'w');
fprintf(savefile,'%s %s %s %s %s','X','Z','SD','CN','Cerr');
fprintf(savefile,'\n%e %e %e %e %e', solXZ);
fclose(savefile);
cd(c_dir_prof)

function select_solutions_2D_Callback(src,eventdata)
global X F m c_dir_prof pname_prof fnameprofile info_line_prof
global psol_i px_sol0 pz_sol0 pz_sol0_SD pc_sol0 per_sol0
clear new_px_sol0 new_pz_sol0 new_pz_sol0_SD new_pc_sol0 new_pc_sol0 new_absper_sol0
% SELECT 2D SOLUTIONS (based on SD, cond. number and Cooper's criterion)

% get the last selection line positions from the axes handle
SD_UD=get(findobj('Tag','SD_p'),'UserData');
sd_str=get(SD_UD.line_position,'String');
[s,r]=strtok(sd_str);[s,sd_str]=strtok(r);

SN_UD=get(findobj('Tag','CN_p'),'UserData');
cn_str=get(SN_UD.line_position,'String');
[s,r]=strtok(cn_str);[s,cn_str]=strtok(r);

CE_UD=get(findobj('Tag','CE_p'),'UserData');
ce_str=get(CE_UD.line_position,'String');
[s,r]=strtok(ce_str);[s,ce_str]=strtok(r);

% SD selection value verification
if str2num(sd_str)<=max(pz_sol0_SD);
   sd_str=sd_str;
else
   sd_str=num2str(max(pz_sol0_SD));
end
% CN selection value verification
if str2num(cn_str)<=max(pc_sol0);
   cn_str=cn_str;
else
   cn_str=num2str(max(pc_sol0));
end
% CE selection value verification
if str2num(ce_str)<=max(abs(per_sol0))
   ce_str=ce_str;
else
   ce_str=num2str(max(abs(per_sol0)))
end

 prompti = {'standard deviation of the solution','condition number of the solution','error of the solution (in Coopers sense)'};
 titlei = 'limits of accepted 2D-Euler solutions (3 criteria acceptation)';
 linesi = 1; 
 defi = {sd_str,cn_str,ce_str};
 
 %povodne
 %defi = {num2str(max(pz_sol0_SD)),num2str(max(pc_sol0)),num2str(max(abs(per_sol0)))};
 
 answer = inputdlg(prompti,titlei,linesi,defi);
 opergstr1 = answer(1); limit_pz_sol0_SD = str2double(opergstr1);
 opergstr2 = answer(2); limit_pc_sol0 = str2double(opergstr2);
 opergstr3 = answer(3); limit_per_sol0 = str2double(opergstr3);

new_psol_i = 0; 
% SELECTION CYCLE
for pi=1:psol_i
    if ( pz_sol0_SD(pi,1)<limit_pz_sol0_SD ) & ( pc_sol0(pi,1)<limit_pc_sol0 ) & ( abs(per_sol0(pi,1))<limit_per_sol0 )
        new_psol_i=new_psol_i + 1;
        new_px_sol0(new_psol_i,1) = px_sol0(pi,1); new_pz_sol0(new_psol_i,1) = pz_sol0(pi,1);
        new_pz_sol0_SD(new_psol_i,1) = pz_sol0_SD(pi,1); new_pc_sol0(new_psol_i,1) = pc_sol0(pi,1);
        new_absper_sol0(new_psol_i,1) = abs(per_sol0(pi,1)); new_pz_sol0_SD(new_psol_i,1) = pz_sol0_SD(pi,1);
    end;
end;
selected_2DEuler_solutions = new_psol_i;
selected_2DEuler_solutions

minnewzsol0 = min(new_pz_sol0);
% plotting of selected solutions
fig_sel_p=figure('Name','selected 2D-Euler depth solutions','NumberTitle','off');
plot(new_px_sol0,new_pz_sol0,'--rs','LineWidth',2,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',6)
axis([X(1) X(m) minnewzsol0 0]);
xlabel('x [dist. units]','FontSize',10); ylabel('depth [dist. units]','FontSize',10);
title({'selected 2D-Euler depth solutions (saved in Euler-2D-solutions-selected.dat file)',info_line_prof});
set(fig_sel_p,'WindowButtonMotionFcn',@cursorcoordinate)

% writing of determined solutions into a file
% creating a matrix from solution vectors
solXZ = [new_px_sol0'; new_pz_sol0'; new_pz_sol0_SD'; new_pc_sol0'; new_absper_sol0'];
cd(pname_prof)
part1='euler_2D_sol_sel_';part2=fnameprofile;nametogether=strcat(part1,part2);
savefile = fopen(nametogether, 'w');
fprintf(savefile,'%s %s %s %s %s','X','Z','SD','CN','Cerr');
fprintf(savefile,'\n%e %e %e %e %e', solXZ);
fclose(savefile);
cd(c_dir_prof)

%4. GRID EULER DECONVOLUTION~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

function Euler_deconvolution_3D_Callback(src,eventdata)
global gridEDwind gridED_Nstr gridED_polyn field
global grid_regdx grid_regdy grid_regdz
global columnsn rowsm minx maxx miny maxy
global xax yax c_dir_grid pname_grid fnamegrid
global sol_i x_sol0 y_sol0 z_sol0 z_sol0_SD c_sol0 er_sol0
global fig_SD_g fig_CN_g fig_CE_g info_line_grid

%Initial check
%data check
if isempty(field)
    helpdlg('Load grid data first','Wrong selection');
    return
end

%derivatives check
if isempty(grid_regdx)||isempty(grid_regdy)||isempty(grid_regdz)
    helpdlg('Compute x, y & z derivatives first','Wrong selection');
    return
end

% INPUT OF THE PARAMETERS OF THE GRID EULER DECONVOLUTION

 prompti = {'grid window size for ED (e.g. 10)','structural index: 0(contact-mag,dike-grav), 1(dike-mag,cylin.-grav),       2(cylin.-mag,sphere-grav), 3(sphere-mag)','interference polynomial degree (0, 1 or 2) [0 for classical approach]'};
 titlei = 'parameters of grid (3D) Euler deconvolution';
 linesi = 1; defi = {num2str(gridEDwind),num2str(gridED_Nstr),num2str(gridED_polyn)};
 answer = inputdlg(prompti,titlei,linesi,defi);
 opergstr1 = answer(1); gridEDwind = str2double(opergstr1);
 opergstr2 = answer(2); gridED_Nstr = str2double(opergstr2);
 opergstr3 = answer(3); gridED_polyn = str2double(opergstr3);
 

%clear old Euler deconv. solutions (if any)
sol_i=[]; x_sol0=[]; y_sol0=[]; z_sol0=[]; N_sol0=[]; z_sol0_SD=[]; c_sol0=[]; er_sol0=[]; 
 
%clear old selection criteria plots and data (if any)
if ishandle(fig_SD_g)
    close(fig_SD_g)
    clear get(findobj('Tag','SD_p_g'),'UserData')
end
if ishandle(fig_CN_g)
   close(fig_CN_g)
   clear get(findobj('Tag','CN_g'),'UserData')
end
if ishandle(fig_CE_g)
    close(fig_CE_g)
    clear get(findobj('Tag','CE_g'),'UserData')
end


% 3D EULER DECONV. ALGORITHM (Reid et al., 1990-equation format [right-hand side: -N(f-B)]) + interference polynomial introduction (Pasteka, 2000)
matrdimx = columnsn; matrdimy = rowsm;
dx = (maxx - minx)/(matrdimx-1); dy = (maxy - miny)/(matrdimy-1);

 for i=1:matrdimx
  XX(i) = minx + dx*(i-1);
 end;
 for j=1:matrdimy 
  YY(j) = miny + dy*(j-1);
 end;
 
operl = gridEDwind; SI = gridED_Nstr; Fx = grid_regdx; Fy = grid_regdy; Fz = grid_regdz;
% MAIN ALGORITHM OF EULER DECONVOLUTION 
% main cycle along the whole grid, sol_i - order number of obtained solutions
sol_i=0;
for i=1:matrdimx-operl-1
   for j=1:matrdimy-operl-1
        % creation of the matrix for the interpr. equation (left-hand side matrix A) and right-hand side vector b
        kl=1;
        for ki=0:operl-1
          for kj=0:operl-1
            switch gridED_polyn
                case 0
                A(kl,1) = Fx(j+kj,i+ki); A(kl,2)=Fy(j+kj,i+ki); A(kl,3)=Fz(j+kj,i+ki); A(kl,4)=SI;
                case 1
                A(kl,1) = Fx(j+kj,i+ki); A(kl,2)=Fy(j+kj,i+ki); A(kl,3)=Fz(j+kj,i+ki); A(kl,4)=1; A(kl,5)=XX(i+ki); A(kl,6)=YY(j+kj); A(kl,7)=XX(i+ki)*YY(j+kj);
                case 2
                A(kl,1) = Fx(j+kj,i+ki); A(kl,2)=Fy(j+kj,i+ki); A(kl,3)=Fz(j+kj,i+ki); A(kl,4)=1; A(kl,5)=XX(i+ki); A(kl,6)=YY(j+kj); A(kl,7)=XX(i+ki)*YY(j+kj);
                A(kl,8)=XX(i+ki)*XX(i+ki); A(kl,9)=YY(j+kj)*YY(j+kj); A(kl,10)=XX(i+ki)*XX(i+ki)*YY(j+kj); A(kl,11)=YY(j+kj)*YY(j+kj)*XX(i+ki);
            end;% switch
          b(kl,1) = SI*field(j+kj,i+ki) + XX(i+ki)*Fx(j+kj,i+ki) + YY(j+kj)*Fy(j+kj,i+ki);
          kl=kl+1;  
          end;
        end; 
        % solution of the Ax = b system - by means of Moore-Penrose method
        % xs = pinv(A)*b;
        % solution of the Ax = b system - by means of the MatLab LSQ approach
        xs = A\b;
        % inverse of the normal equation matrix (for the evaluation of the stand. deviation of the solution)
        iA = inv(A'*A);
        % evaluation of the condition number of the solution
        c = cond(A);
        % number of solutions will be increased
        sol_i = sol_i + 1;
        % calculation of parameters x0, y0 and z0
        x_sol0(sol_i,1) = xs(1); x_sol0_SD(sol_i,1) = sqrt(iA(1,1));
        y_sol0(sol_i,1) = xs(2); y_sol0_SD(sol_i,1) = sqrt(iA(2,2));
        z_sol0(sol_i,1) = -xs(3); z_sol0_SD(sol_i,1) = sqrt(iA(3,3));
        B_sol0(sol_i,1) = xs(4)/(SI+1E-6);
        % evaluation of the condition number of the solution
        c_sol0(sol_i,1) = c;
        % evaluation of the ED-equation error (based on Cooper,2004 idea)
		switch gridED_polyn
                case 0
                sumer = 0;
                for ki=0:operl-1
                    for kj=0:operl-1
                    sumer = sumer + xs(1)*Fx(j+kj,i+ki)+xs(2)*Fy(j+kj,i+ki)+xs(3)*Fz(j+kj,i+ki)+SI*xs(4)-XX(i+ki)*Fx(j+kj,i+ki)-YY(j+kj)*Fy(j+kj,i+ki)-SI*field(j+kj,i+ki);
                    end;
                end; er_sol0(sol_i,1) = sumer;
                case 1
                sumer = 0;
                for ki=0:operl-1
                    for kj=0:operl-1
                    sumer = sumer + xs(1)*Fx(j+kj,i+ki)+xs(2)*Fy(j+kj,i+ki)+xs(3)*Fz(j+kj,i+ki)+SI*xs(4)+SI*xs(5)*XX(i+ki)+SI*xs(6)*YY(j+kj)+SI*xs(7)*XX(i+ki)*YY(j+kj)-XX(i+ki)*Fx(j+kj,i+ki)-YY(j+kj)*Fy(j+kj,i+ki)-SI*field(j+kj,i+ki);
                    end;
                end; er_sol0(sol_i,1) = sumer;
                case 2
                sumer = 0;
                for ki=0:operl-1
                    for kj=0:operl-1
                    sumer = sumer + xs(1)*Fx(j+kj,i+ki)+xs(2)*Fy(j+kj,i+ki)+xs(3)*Fz(j+kj,i+ki)+SI*xs(4)+SI*xs(5)*XX(i+ki)+SI*xs(6)*YY(j+kj)+SI*xs(7)*XX(i+ki)*YY(j+kj)+SI*xs(8)*XX(i+ki)*XX(i+ki)+SI*xs(9)*YY(j+kj)*YY(j+kj)+SI*xs(10)*XX(i+ki)*XX(i+ki)*YY(j+kj)+SI*xs(11)*XX(i+ki)*YY(j+kj)*YY(j+kj)-XX(i+ki)*Fx(j+kj,i+ki)-YY(j+kj)*Fy(j+kj,i+ki)-SI*field(j+kj,i+ki);
                    end;
                end; er_sol0(sol_i,1) = sumer;
        end; 
        % non-accepted if the solution for x0 and y0 is not in the frame of the solution-window and when z-solution is positive
		 if (x_sol0(sol_i,1)<XX(i)) | (x_sol0(sol_i,1)>XX(i+operl-1)) | (y_sol0(sol_i,1)<YY(j)) | (y_sol0(sol_i,1)>YY(j+operl-1)) | ( imag(z_sol0(sol_i,1))~=0 ) | (z_sol0(sol_i,1)>0)
            sol_i=sol_i - 1;
         end;
	%sol_i
    end; % end of the main cycle along the profile data (j),      
 end; % end of the main cycle along the profile data (i),
 all_3DEuler_solutions = sol_i;
 all_3DEuler_solutions
 
 if sol_i > 0
 x_sol0(sol_i,1) = x_sol0(sol_i-1,1); y_sol0(sol_i,1) = y_sol0(sol_i-1,1);
 z_sol0(sol_i,1) = z_sol0(sol_i-1,1); c_sol0(sol_i,1) = c_sol0(sol_i-1,1);
 er_sol0(sol_i,1) = er_sol0(sol_i-1,1); z_sol0_SD(sol_i,1) = z_sol0_SD(sol_i-1,1);
 end;

% plotting of obtained standard deviations 
fig_SD_g=figure('Name','standard deviation of the depth solution','NumberTitle','off','Tag','SD_fig');
scatter(x_sol0,z_sol0_SD,25,'MarkerEdgeColor','k','MarkerFaceColor','r');grid on;
axis([XX(1) XX(matrdimx) min(z_sol0_SD) max(z_sol0_SD)]); 
xlabel('x [dist.units]','FontSize',10); ylabel('standard deviation of the depth solution [dist.units]','FontSize',10);title('selection of 3D-Euler solutions (standard deviation)');
set(fig_SD_g,'WindowButtonMotionFcn',@cursorcoordinate);
selectionLine(fig_SD_g);
set(gca(fig_SD_g),'Tag','SD_g');

% plotting of obtained condition numbers 
fig_CN_g=figure('Name','condition numbers of the depth solution','NumberTitle','off','Tag','CN_fig');
scatter(x_sol0,c_sol0,25,'MarkerEdgeColor','k','MarkerFaceColor','b');grid on;
axis([XX(1) XX(matrdimx) min(c_sol0) max(c_sol0)]); 
xlabel('x [dist.units]','FontSize',10); ylabel('condition number of the depth solution []','FontSize',10);title('selection of 3D-Euler solutions (condition number)');
set(fig_CN_g,'WindowButtonMotionFcn',@cursorcoordinate);
selectionLine(fig_CN_g);
set(gca(fig_CN_g),'Tag','CN_g');

% plotting of obtained "Cooper's errors"
fig_CE_g=figure('Name','errors in Cooper''s sense','NumberTitle','off','Tag','CE_fig');
scatter(x_sol0,abs(er_sol0),25,'MarkerEdgeColor','k','MarkerFaceColor','m');grid on;
axis([XX(1) XX(matrdimx) min(abs(er_sol0)) max(abs(er_sol0))]); 
xlabel('x [dist.units]','FontSize',10); ylabel('fit errors (in Coopers sense) of the depth solutions [nT (or) mGal]','FontSize',10);title('selection of 3D-Euler solutions (solution fit)');
set(fig_CE_g,'WindowButtonMotionFcn',@cursorcoordinate);
selectionLine(fig_CE_g);
set(gca(fig_CE_g),'Tag','CE_g');


% plotting of 3D-Euler solutions 
fig_sol_g=figure('Name','3D-Euler depth solutions','NumberTitle','off');
contour(xax,yax,field),colormap(gca(fig_sol_g),colscale)
hold on
scatter(x_sol0,y_sol0,25,'MarkerEdgeColor','k','MarkerFaceColor','k')

axis([XX(1) XX(matrdimx) YY(1) YY(matrdimy)]); xlabel('x [dist.units]','FontSize',10); ylabel('y[dist.units]','FontSize',10);
info_line_grid = sprintf('ED parameters: Window size: %.0f,Structural index: %.0f, Polynomial: %.0f',gridEDwind,gridED_Nstr,gridED_polyn);
title({'all 3D-Euler solutions (file:Euler-3D-solutions-all.dat)',info_line_grid});
grid on
set(fig_sol_g,'WindowButtonMotionFcn',@cursorcoordinate)

% writing of determined solutions into a file
% creating a matrix from solution vectors
solXZ = [x_sol0'; y_sol0'; z_sol0'; z_sol0_SD'; c_sol0'; abs(er_sol0)';];

cd (pname_grid)
part1='euler_3D_sol_all_';part2=strrep(fnamegrid,'.grd','');part3= '.dat';nametogether=strcat(part1,part2,part3);
savefile = fopen(nametogether, 'w');
fprintf(savefile,'%s %s %s %s %s %s','X','Y','Z','SD','CN','Cerr');
fprintf(savefile,'\n%e %e %e %e %e %e', solXZ);
fclose(savefile);
cd (c_dir_grid)

function select_solutions_3D_Callback(src,eventdata)
global columnsn rowsm minx maxx miny maxy field xax yax c_dir_grid pname_grid fnamegrid
global sol_i x_sol0 y_sol0 z_sol0 z_sol0_SD c_sol0 er_sol0 info_line_grid
global new_x_sol0 new_y_sol0 new_z_sol0
global new_z_sol0_SD new_c_sol0 new_abser_sol0


%clear old selected Euler deconv. solutions (if any)
new_x_sol0=[];new_y_sol0=[];new_z_sol0=[];
new_z_sol0_SD=[];new_c_sol0=[];new_abser_sol0=[];


% SELECT 3D SOLUTIONS (based on SD, cond. number and Cooper's criterion)
% get the last selection line positions from the axes handle
SD_UD_g=get(findobj('Tag','SD_g'),'UserData');
sd_str_g=get(SD_UD_g.line_position,'String');
[s,r]=strtok(sd_str_g);[s,sd_str_g]=strtok(r);

SN_UD_g=get(findobj('Tag','CN_g'),'UserData');
cn_str_g=get(SN_UD_g.line_position,'String');
[s,r]=strtok(cn_str_g);[s,cn_str_g]=strtok(r);

CE_UD_g=get(findobj('Tag','CE_g'),'UserData');
ce_str_g=get(CE_UD_g.line_position,'String');
[s,r]=strtok(ce_str_g);[s,ce_str_g]=strtok(r);

% SD selection value verification
if str2num(sd_str_g)<=max(z_sol0_SD);
   sd_str_g=sd_str_g;
else
   sd_str_g=num2str(max(z_sol0_SD));
end
% CN selection value verification
if str2num(cn_str_g)<=max(c_sol0);
   cn_str_g=cn_str_g;
else
   cn_str_g=num2str(max(c_sol0));
end
% CE selection value verification
if str2num(ce_str_g)<=max(abs(er_sol0))
   ce_str_g=ce_str_g;
else
   ce_str_g=num2str(max(abs(er_sol0)))
end

 prompti = {'standard deviation of the solution','condition number of the solution','error of the solution (in Coopers sense)'};
 titlei = 'limits of accepted 3D-Euler solutions (3 criteria acceptation)';
 linesi = 1; 
 
  defi = {sd_str_g,cn_str_g,ce_str_g};
 
 %defi = %{num2str(max(z_sol0_SD)),num2str(max(c_sol0)),num2str(max(abs(er_sol0)))}
 %povodna definicia z maxim osi
  
 answer = inputdlg(prompti,titlei,linesi,defi);
 
 opergstr1 = answer(1); limit_z_sol0_SD = str2double(opergstr1);
 opergstr2 = answer(2); limit_c_sol0 = str2double(opergstr2);
 opergstr3 = answer(3); limit_er_sol0 = str2double(opergstr3);

new_sol_i = 0; 
% SELECTION CYCLE
for pi=1:sol_i
    if ( z_sol0_SD(pi,1)<limit_z_sol0_SD ) & ( c_sol0(pi,1)<limit_c_sol0 ) & ( abs(er_sol0(pi,1))<limit_er_sol0 )
        new_sol_i=new_sol_i + 1;
        new_x_sol0(new_sol_i,1) = x_sol0(pi,1); new_y_sol0(new_sol_i,1) = y_sol0(pi,1); new_z_sol0(new_sol_i,1) = z_sol0(pi,1);
        new_z_sol0_SD(new_sol_i,1) = z_sol0_SD(pi,1); new_c_sol0(new_sol_i,1) = c_sol0(pi,1);
        new_abser_sol0(new_sol_i,1) = abs(er_sol0(pi,1)); new_z_sol0_SD(new_sol_i,1) = z_sol0_SD(pi,1);
    end;
end;
selected_3DEuler_solutions = new_sol_i;
selected_3DEuler_solutions

% plotting of selected solutions 
matrdimx = columnsn; matrdimy = rowsm; dx = (maxx - minx)/(matrdimx-1); dy = (maxy - miny)/(matrdimy-1);
 for i=1:matrdimx
  XX(i) = minx + dx*(i-1);
 end;
 for j=1:matrdimy 
  YY(j) = miny + dy*(j-1);
 end;
minnewzsol0 = min(new_z_sol0);

% XZ slice
fig_sel_g_xz=figure('Name','selected 3D-Euler depth solutions','NumberTitle','off','WindowButtondownFcn',@dragret);
scatter(new_x_sol0,new_z_sol0,25,'MarkerEdgeColor','k','MarkerFaceColor','g')
grid on;
axis([XX(1) XX(matrdimx) minnewzsol0 0]); xlabel('x [dist. units]','FontSize',10); ylabel('depth [dist. units]','FontSize',10);
title({'selected 3D-Euler solutions (depth view)',info_line_grid});           

scr=get(0,'ScreenSize');pos=get(fig_sel_g_xz,'Position');out_pos=get(fig_sel_g_xz,'OuterPosition');
set(fig_sel_g_xz,'Position',[scr(3)/2-out_pos(3) (scr(4)/4)+20 pos(3) pos(4)],'WindowButtonMotionFcn',@cursorcoordinate)

% Area map of solutions
fig_sel_g_xy=figure('Name','selected 3D-Euler depth solutions in area','NumberTitle','off','WindowButtondownFcn',@dragret);
contour(xax,yax,field),colormap(gca(fig_sel_g_xy),colscale)
hold on
scatter(new_x_sol0,new_y_sol0,25,'MarkerEdgeColor','k','MarkerFaceColor','g')
               
axis([XX(1) XX(matrdimx) YY(1) YY(matrdimy)]); xlabel('x [dist. units]','FontSize',10); ylabel('y[dist]','FontSize',10);
title({'selected 3D-Euler solutions (file: Euler-3D-solutions-selected.dat)',info_line_grid});
grid on
pos=get(fig_sel_g_xy,'Position');
set(fig_sel_g_xy,'Position',[scr(3)/2 (scr(4)/4)+20 pos(3) pos(4)],'WindowButtonMotionFcn',@cursorcoordinate)

% writing of determined solutions into a file
% creating a matrix from solution vectors
solXZ = [new_x_sol0'; new_y_sol0'; new_z_sol0'; new_z_sol0_SD'; new_c_sol0'; new_abser_sol0'];
cd (pname_grid)
part1='euler_3D_sol_sel_';part2=strrep(fnamegrid,'.grd','');part3= '.dat';nametogether=strcat(part1,part2,part3);
savefile = fopen(nametogether, 'w');
fprintf(savefile,'%s %s %s %s %s %s','X','Y','Z','SD','CN','Cerr');
fprintf(savefile,'\n%e %e %e %e %e %e', solXZ);
fclose(savefile);
cd (c_dir_grid)


%4. ABOUT~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

function about_Callback(src,eventdata)
% ABOUT
msgbox('                           REGDER2_0v2, 2010                                      Christian-Albrechts-University Kiel, Germany                                                  Comenius University Bratislava, Slovakia                                                 Authors: R.Pasteka & D.Kusnirak,   pasteka@fns.uniba.sk','About')

function matlab_exit_Callback(src,eventdata)
clear all;close all;clc


%=====================ADDITIONAL FUNCTIONS=================================
function [g,PointExt] = cosxp2prof(T,perc)
%Expand profile using cosine taper mirror function, 'perc' (%) expansion. 
%where perc is a decimal that represents the expansion %, e.g. 0.15 --> 15% 
%g - final extrapolated field; PointExt - number of the added points on one edge
%Written by: Daniela Gerovska and Marcos Arauzo-Bravo (27/February/2003) 
if ~exist('perc')
    perc=0.1;   % default exapnsion is 10%
end
perc = 1+perc;

NumPoints = numel(T);
%Deciding NumPointsExt NumColExt
NumPointsExt=Eve10Ext(NumPoints,perc);
g=T;
%Extention of the grid with half a cosine function
% Cosine taper right side
lim=NumPointsExt-NumPoints; rlamb=lim+1; arg=pi/rlamb;
s=0.5*(g(NumPoints)+g(1)); r=0.5*(g(NumPoints)-g(1));
for k=1:lim
    ge=s+r*cos(arg*k);
    g(NumPoints+k)=ge;
end
% Cosine taper left side
g = flipud(g);
lim=NumPointsExt-NumPoints; rlamb=lim+1; arg=pi/rlamb;
s=0.5*(g(NumPointsExt)+g(lim+1)); r=0.5*(g(NumPointsExt)-g(lim+1));
for k=1:lim
    ge=s+r*cos(arg*k);
    g(NumPointsExt+k)=ge;
end
g = flipud(g);
PointExt = NumPointsExt - NumPoints;

function [g,RowExt,ColExt] = cosxp2(T,perc)
%Expand grid using cosine taper mirror function, 'perc' (%) expansion. 
%where perc is a decimal that represents the expansion %, e.g. 0.15 --> 15% 
%g - final extrapolated field; RowExt - number of the added rows on one edge, ColExt - the same for columns
%Written by: Daniela Gerovska and Marcos Arauzo-Bravo (27/February/2003) 

if ~exist('perc')
    perc=0.1;   % default exapnsion is 10%
end
perc = 1+perc;

[NumRow,NumCol]=size(T);
%Deciding NumRowExt NumColExt
NumRowExt=Eve10Ext(NumRow,perc);
NumColExt=Eve10Ext(NumCol,perc);
g=T;
%Extention of the grid with half a cosine function
% Cosine taper right side
lim=NumColExt-NumCol; rlamb=lim+1; arg=pi/rlamb;
s=0.5*(g(:,NumCol)+g(:,1)); r=0.5*(g(:,NumCol)-g(:,1));
for k=1:lim
    ge=s+r*cos(arg*k);
    g(:,NumCol+k)=ge;
end
% Cosine taper left side
g = fliplr(g);
lim=NumColExt-NumCol; rlamb=lim+1; arg=pi/rlamb;
s=0.5*(g(:,NumColExt)+g(:,lim+1)); r=0.5*(g(:,NumColExt)-g(:,lim+1));
for k=1:lim
    ge=s+r*cos(arg*k);
    g(:,NumColExt+k)=ge;
end
g = fliplr(g);
% Cosine taper upper side
g=g';
lim=NumRowExt-NumRow; rlamb=lim+1; arg=pi/rlamb;
s=0.5*(g(:,NumRow)+g(:,1)); r=0.5*(g(:,NumRow)-g(:,1));
for k=1:lim
    ge=s+r*cos(arg*k);
    g(:,NumRow+k)=ge;
end
g=g';
% Cosine taper lower side
g = flipud(g); g=g';
lim=NumRowExt-NumRow; rlamb=lim+1; arg=pi/rlamb;
s=0.5*(g(:,NumRowExt)+g(:,lim+1)); r=0.5*(g(:,NumRowExt)-g(:,lim+1));
for k=1:lim
    ge=s+r*cos(arg*k);
    g(:,NumRowExt+k)=ge;
end
g=g'; g = flipud(g);
RowExt = NumRowExt - NumRow; ColExt = NumColExt - NumCol;

function NumEveExt=Eve10Ext(Num,perc)
%Even 'perc'(%) extension of the number Num
%Written by: Daniela Gerovska and Marcos Arauzo-Bravo (27/February/2003) 
 NumExt=perc*Num;
NumExtRou=round(NumExt);
if rem(NumExtRou,2)==1 
  DifPlu=abs(NumExt-(NumExtRou+1));
  DifMin=abs(NumExt-(NumExtRou-1));
    if DifPlu>DifMin
       NumEveExt=NumExtRou-1;
    else 
       NumEveExt=NumExtRou+1;
    end%if DifPlu>DifMin  
else
   NumEveExt=NumExtRou;   
end %if rem(NumExtRou,2)==1  

function cursorcoordinate(src,eventdata)

% find axes associated with calling window
hax = gca(src);     

% Axes limits
xlims = get(hax, 'XLim');
ylims = get(hax, 'YLim');

% format for x coordinate
if max(abs(xlims))>999999       %exponencial format definition for values over 10^6
   xformat_str          = 'X: %.2e,';
elseif max(abs(xlims))<0.1      %more precise format definition for values under 0.1
   xformat_str          = 'X: %.4f,';
else                            %standard format with 2 decimals
   xformat_str          = 'X: %.2f,';
end

% format for y coordinate
if max(abs(ylims))>999999
   yformat_str          = ' Y: %.2e';
elseif max(abs(ylims))<0.1
   yformat_str          = ' Y: %.4f';
else
   yformat_str          = ' Y: %.2f';
end

format_str=strcat(xformat_str,yformat_str);

% initial text string
curr_position   = get(hax, 'CurrentPoint');
position        = [curr_position(1, 1) curr_position(1, 2)];
position_string = sprintf(format_str, position); %position string as a text


% do not update if current mouse position is out of axes 
in_bounds = (position(1) >= xlims(1)) && ...
            (position(1) <= xlims(2)) && ...
            (position(2) >= ylims(1)) && ...
            (position(2) <= ylims(2));

if (~in_bounds)  
    return
end        

user_data = get(src, 'UserData'); 

%info text inicialization during the first callback call and its save 
%to the axes handle as as a UserData. Units were set to normalized to
%keep the relative position with axes

if ((~isfield(user_data, 'cursorcoordinate')) || ~ishandle(user_data.cursorcoordinate))

    user_data.cursorcoordinate = text(0, 0, 'blabla','Units','normalized','Parent',hax);

    set(src, 'UserData', user_data); % save to axes
end


%'online' text update
set(user_data.cursorcoordinate, 'Position', [0 -0.1],...
                           'String', position_string,...
                           'VerticalAlignment', 'bottom',...
                           'HorizontalAlignment', 'left',...
                           'Parent',hax);

function selectionLine(src,eventdata)

% Initializations
fig=src;        %figure handle 
curS = [];      %current selection line handle
hax = gca(fig); %axes handl;e


% Get axes limits
xlims = xlim(hax);
ylims = ylim(hax);
ypos = mean(ylim(hax));

%fixed axes scale (not necessary)
axis ([xlims(1) xlims(2) ylims(1) ylims(2)])

% Create selection line in the middle (ypos=mean of y limits)
Sline = line(xlims,[ypos ypos],'Color','m','LineWidth',2,'Parent',hax);

% Initial position string (defined in slighlty different way because of
% using additional callbacks)

if max(abs(ylims))>999999     %exponencial format definition for values over 10^6
format_str = 'selection criteria: %.3e';
elseif max(abs(ylims))<0.1    %more precise format definition for values under 0.1
format_str = 'selection criteria: %.3e';
else                          %standard format with 2 decimals
format_str = 'selection criteria: %.2f';
end
line_position_value  = ypos;
line_position_string = sprintf(format_str, line_position_value); %text string

user_data = get(hax, 'UserData'); 
user_data.line_position = text(0, -0.1 ,'','Units','normalized');

set(hax, 'UserData', user_data); % save to axes

%plot to the figure on the defined postion
set(user_data.line_position,'Position', [1, -0.1],...
                       'String', line_position_string,...
                       'VerticalAlignment', 'bottom',...
                       'HorizontalAlignment', 'right')
%                        ,...
%                            'Tag','SelLine');


% ButtonDownFcn Callback - function called if the inicial line is clicked
set(Sline,'ButtonDownFcn',{@ButtonDownFcn,user_data,fig,format_str})

function ButtonDownFcn(src,eventdata,user_data,fig,format_str)

% Update current selection line handle
curS=src;

%custom mouse cursor definition
cursor = [...
NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
NaN   NaN   NaN   NaN   NaN   NaN   NaN     2   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
NaN   NaN   NaN   NaN   NaN   NaN     2     1     2   NaN   NaN   NaN   NaN   NaN   NaN   NaN
NaN   NaN   NaN   NaN   NaN     2     1     1     1     2   NaN   NaN   NaN   NaN   NaN   NaN
NaN   NaN   NaN   NaN     2     1     1     1     1     1     2   NaN   NaN   NaN   NaN   NaN
NaN   NaN   NaN     2     1     1     1     1     1     1     1     2   NaN   NaN   NaN   NaN
 2     2     2     2     2     2     1     1     1     2     2     2     2     2     2   NaN
 2     1     1     1     1     1     1     1     1     1     1     1     1     1     2   NaN
 2     2     2     2     2     2     2     2     2     2     2     2     2     2     2   NaN
 2     1     1     1     1     1     1     1     1     1     1     1     1     1     2   NaN
 2     2     2     2     2     2     1     1     1     2     2     2     2     2     2   NaN
NaN   NaN   NaN     2     1     1     1     1     1     1     1     2   NaN   NaN   NaN   NaN
NaN   NaN   NaN   NaN     2     1     1     1     1     1     2   NaN   NaN   NaN   NaN   NaN
NaN   NaN   NaN   NaN   NaN     2     1     1     1     2   NaN   NaN   NaN   NaN   NaN   NaN
NaN   NaN   NaN   NaN   NaN   NaN     2     1     2   NaN   NaN   NaN   NaN   NaN   NaN   NaN
NaN   NaN   NaN   NaN   NaN   NaN   NaN     2   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN];

%cursor active point (in the middle)
cursorHotSpot = [8 8];

% pointer change
set(fig,'Pointer','custom','PointerShapeCData',cursor,'PointerShapeHotSpot',cursorHotSpot)


% Callbacks for cases- mouse is still clicked and is moving
%                    - mouse button have been unclicked    
set(fig,'WindowButtonMotionFcn',{@ButtonMotionFcn,curS,user_data,format_str})
set(fig,'WindowButtonUpFcn',@ButtonUpFcn)

function ButtonMotionFcn(src,eventdata,curS,user_data,format_str)

% Get current point
hax=gca(src);
curP = get(hax,'CurrentPoint');

% axes limits defined again to avoid passing them through callbacks
xlims = xlim(hax);
ylims = ylim(hax);
axis ([xlims(1) xlims(2) ylims(1) ylims(2)])

% update selection line position in order to the axes limits
if curP(1,2) > ylims(1) && curP(1,2) < ylims(2)

    set(curS,'XData',xlims,'YData',[curP(1,2) curP(2,2)])

else
    
    return

end

%'online' text update correlating with actual line position
line_position_value  = curP(1,2);
line_position_string = sprintf(format_str, line_position_value);

set(user_data.line_position, 'Position', [1, -0.1],...
                            'String', line_position_string,...
                            'VerticalAlignment', 'bottom',...
                            'HorizontalAlignment', 'right')
%                           ,...
%                           'Tag','SelLine');

% refresh display
drawnow;

function ButtonUpFcn(src,eventdata)

fig=src;
% change the cursor pointer
set(fig,'Pointer','arrow')

% activating the cursor coordinate function 
set(fig,'WindowButtonMotionFcn',@cursorcoordinate)

function [scale]=colscale()

% clr file import
fid = fopen('REGDER2_0.clr');
A = fscanf(fid,'%s',1);
B = fscanf(fid,'%f',2);
C = fscanf(fid,'%f');
fclose(fid);

%colormap conversion to the matlab format
D = (reshape(C,4,numel(C)/4))';
scale=((D(:,2:4))/255);

function normpick_2D(src,eventdata)
global optalphaz optalphax
global optalphaxg optalphayg optalphazg

fig=src; 
cur_char=get(src,'CurrentKey');
set(fig,'WindowButtonMotionFcn',@cursorcoordinate)

if cur_char=='Z' || cur_char=='z'
   
    for i=1:1000
       set(fig,'pointer','Fullcross')
       k=waitforbuttonpress;
       
       if k==0
           button=get(fig,'SelectionType');
           cor=get(gca(fig),'CurrentPoint');
           corx=cor(1,1);cory=cor(1,2);
           cor_i(i,1:3)=cor(1,:);
           if strcmp(button, 'alt')
              corx=0;
              if i==1
                 cor_i(1,1:3)=0;
              end
              cor_i(i,1:3)=cor_i(i-1,1:3);
           end
           set(gca(fig),'NextPlot','add')
           scatter(gca(fig),cor_i(:,1),cor_i(:,2),5,'+k')
            switch get(src,'Name')
                case ('L norms for the x-derivative')
                    x_deriv_2D(optalphax,corx);
                case ('L norms for the z-derivative')
                    z_deriv_2D(optalphaz,corx);
                case ('L norms for the x-derivative(3D)')
                    x_deriv_3D(optalphaxg,corx);
                case ('L norms for the y-derivative(3D)')
                    y_deriv_3D(optalphayg,corx);
                case ('L norms for the z-derivative(3D)')
                    z_deriv_3D(optalphazg,corx);
            end
            
       elseif k==1
           set(fig,'pointer','arrow')
           refresh(fig)
           break
           return
       end
   end
end


function dragret(src, eventdata)
global ret hgl_scat hgl_scat_SD hgl_scat_CN hgl_scat_CE
global new_z_sol0_SD new_c_sol0 new_abser_sol0


%find pairs and get data
switch get(src,'Name')
    case ('selected 3D-Euler depth solutions')
    source=(findobj('Name','selected 3D-Euler depth solutions'));
    out=(findobj('Name','selected 3D-Euler depth solutions in area'));
            
        if ishandle(src+1)
           source=source(source==src);
           out=src+1;
        else    
            disp('missing pair')
            return
        end
        
    srcdat=get(gca(source),'Children');outdat=get(gca(out),'Children');    
    
    x_hgl= get(srcdat(end),'Xdata');
    if iscell(x_hgl);x_hgl=x_hgl{end};end
    y_hgl =get(outdat(end-1),'Ydata');
    if iscell(y_hgl);y_hgl=y_hgl{end};end
    z_hgl =get(srcdat(end),'Ydata');
    if iscell(z_hgl);z_hgl=z_hgl{end};end
    xax=x_hgl;yax=z_hgl;

    case ('selected 3D-Euler depth solutions in area')
    source=(findobj('Name','selected 3D-Euler depth solutions in area'));
    out=(findobj('Name','selected 3D-Euler depth solutions'));
        
        if     ishandle(src-1)
               source=source(source==src);
               out=source-1;
        else
            disp('missing pair')
            return
        end
   
    srcdat=get(gca(source),'Children');outdat=get(gca(out),'Children');
    
    x_hgl =get(srcdat(end-1),'Xdata');
    if iscell(x_hgl);x_hgl=x_hgl{end};end
    y_hgl =get(srcdat(end-1),'Ydata');
    if iscell(y_hgl);y_hgl=y_hgl{end};end
    z_hgl =get(outdat(end),'Ydata');
    if iscell(z_hgl);z_hgl=z_hgl{end};end
    xax=x_hgl;yax=y_hgl;

end  

% Drag the retangle
point1 = get(gca,'CurrentPoint');    % button down detected
rbbox                                % start box drawing
point2 = get(gca,'CurrentPoint');    % button up detected
point1 = point1(1,1:2);              % extract x and y
point2 = point2(1,1:2);
p1 = min(point1,point2);             % calculate LL point location
p2 = max(point1,point2);             % calculate TR point location
offset = abs(point1-point2);         % and box dimensions
x = [p1(1) p1(1)+offset(1) p1(1)+offset(1) p1(1) p1(1)];
y = [p1(2) p1(2) p1(2)+offset(2) p1(2)+offset(2) p1(2)];

    
if ishandle(ret)
    if ishandle(get(ret,'parent')) & gca(source)~=get(ret,'parent')
%       disp('diff source')
        delete(ret)
        hold on,axis manual
        ret=plot(x,y,'.-k','Tag','rettag');hold off
    else
        set(ret,'XData',x,'YData',y)
    end
else
    hold on,axis manual
    ret=plot(x,y,'.-k','Tag','rettag');hold off
end


%filer the data within the rectangle
fil_sol=1;xfil=[];yfil=[];zfil=[];
for i=1:length(x_hgl)
    if (xax(i)>p1(1) & xax(i)<p2(1))&(yax(i)>p1(2) & yax(i)<p2(2));
        xfil(fil_sol)=x_hgl(i);
        yfil(fil_sol)=y_hgl(i);
        zfil(fil_sol)=z_hgl(i);
        
        SD_fil(fil_sol)=new_z_sol0_SD(i);
        CN_fil(fil_sol)=new_c_sol0(i);
        CE_fil(fil_sol)=new_abser_sol0(i);
        
        fil_sol=fil_sol+1;
    end
end
highlited_solutions_count=fil_sol-1;


switch get(src,'Name')
    case ('selected 3D-Euler depth solutions')
        xfilpl=xfil;yfilpl=yfil;
    case ('selected 3D-Euler depth solutions in area')
        xfilpl=xfil;yfilpl=zfil;
end


% highlight selected solutions positions
if ishandle(hgl_scat)
    if  ishandle(get(hgl_scat,'parent')) & gca(out)~=get(hgl_scat,'parent')
        delete(hgl_scat)
        axes(gca(out));hold on;
        hgl_scat=scatter(gca(out),xfilpl,yfilpl,25,'MarkerEdgeColor','k','MarkerFaceColor','r');hold off  
    else
    set(hgl_scat,'XData',xfilpl,'YData',yfilpl)
    end
else
    axes(gca(out));
    hold on;
    hgl_scat=scatter(gca(out),xfilpl,yfilpl,25,'MarkerEdgeColor','k','MarkerFaceColor','r');hold off
end

% highlight selected solutions parameters

if isempty(findobj('Tag','SD_fig')) | isempty(findobj('Tag','CN_fig')) | isempty(findobj('Tag','CE_fig'))
    disp ('missing solutions parameter figure, highlighting not functional')
    return
else 
if ishandle(hgl_scat_SD) & ishandle(hgl_scat_CN) & ishandle(hgl_scat_CE)
     
    set(hgl_scat_SD,'XData',xfilpl,'YData',SD_fil)
    set(hgl_scat_CN,'XData',xfilpl,'YData',CN_fil)
    set(hgl_scat_CE,'XData',xfilpl,'YData',CE_fil)
    
else
    if ishandle(hgl_scat_SD)|ishandle(hgl_scat_CN)|ishandle(hgl_scat_CE)
        delete(hgl_scat_SD,hgl_scat_CN,hgl_scat_CE)
    end

    axes(gca((findobj('Tag','SD_fig'))));hold on
    hgl_scat_SD=scatter(gca,xfilpl,SD_fil,25,'MarkerEdgeColor','k','MarkerFaceColor','y');hold off
    
    axes(gca((findobj('Tag','CN_fig'))));hold on
    hgl_scat_CN=scatter(gca,xfilpl,CN_fil,25,'MarkerEdgeColor','k','MarkerFaceColor','y');hold off
    
    axes(gca((findobj('Tag','CE_fig'))));hold on
    hgl_scat_CE=scatter(gca,xfilpl,CE_fil,25,'MarkerEdgeColor','k','MarkerFaceColor','y');hold off
    
end
end
% %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~